c *******************************************************************
c  This program selects objects: galaxies/stars/qsos on the basis of their 
c  xi2 value obtained through the hyperz pipelines
c *******************************************************************
      PROGRAM hyperz_classify
c     Version: 26/11/2002
      IMPLICIT NONE
      INTEGER i,j,k,nparam,input,iblank,n_head,nobj1,nobj2,nobj3,nobj4,
     &     nfilt,nobs,last,n_tg,n_tq,n_ts,id
      PARAMETER (nparam=8)
      REAL pgal,pqso,pstar,p1,p2,zgal,zqso,zstar,c2gal,c2qso,c2star,
     &     amag(50),amerr(50)
      LOGICAL header
      CHARACTER*5000 header_line,linez
      CHARACTER*160 filein,filecat,filegal,fileqso,filestar,fileout,line
      CHARACTER*40 cname,param(nparam),cc*1
      DIMENSION input(nparam)
c ------------------------------------------------------------------------
 11   filein='hyperz_classify.param'
      last=INDEX(filein,'  ')-1
      WRITE(*,*) 'Name of parameter file'
      WRITE(*,*) '[default = ',filein(1:last),']'
      READ(*,'(A)') line
      IF (line.ne.' ') THEN
         READ(line,'(A)',ERR=11) filein
      END IF
c ------------------------------------------------------------------------
c                       Reading parameters file 
c ------------------------------------------------------------------------

      param(1)='CATALOG_FILE'
      param(2)='GALAXY_FILE'
      param(3)='QSO_FILE'
      param(4)='STAR_FILE'
      param(5)='OUTPUT_FILE'
      param(6)='PROB_1'
      param(7)='PROB_2'
      param(8)='N_FILTER'

      DO i=1,nparam
         input(i)=0
      END DO
      OPEN(77,FILE=filein,STATUS='old',ERR=11)
      WRITE(*,'(A)') ' Reading parameter file...'

 7    READ(77,'(A)',ERR=10,END=10) line
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            GO TO 8
         END IF
      END DO
      GO TO 7  ! blank line
 8    IF (line(1:1).eq.'#') GO TO 7 ! comment line
      iblank=INDEX(line,' ')
      cname=line(1:iblank-1) ! variable name
      line=line(iblank:)
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            IF (line(1:1).eq.'#') THEN
               WRITE(*,*) 'Missed value of variable ', cname
               STOP
            END IF
            GO TO 9
         END IF
      END DO
      WRITE(*,*) 'Missed value of variable ', cname
      STOP
 9    iblank=INDEX(line,' ')
      line=line(1:iblank-1) ! part of string containing value of variable
c Parameters in common hyperz & make_catalog
c ..... Photometric catalogue file
      IF (cname.eq.param(1)) THEN
         READ(line,'(A)') filecat
         input(1)=1
         GO TO 7
c ..... Photometric redshift file (galaxies)
      ELSE IF (cname.eq.param(2)) THEN
         READ(line,'(A)') filegal
         input(2)=1
         GO TO 7
c ..... Photometric redshift file (QSO)
      ELSE IF (cname.eq.param(3)) THEN
         READ(line,'(A)') fileqso
         input(3)=1
         GO TO 7
c ..... Photometric redshift file (star)
      ELSE IF (cname.eq.param(4)) THEN
         READ(line,'(A)') filestar
         input(4)=1
         GO TO 7
c ..... Output total file
      ELSE IF (cname.eq.param(5)) THEN
         READ(line,'(A)') fileout
         input(5)=1
         GO TO 7
c ..... Low P(chi2)
      ELSE IF (cname.eq.param(6)) THEN
         READ(line,*) p1
         input(6)=1
         GO TO 7
c ..... High P(chi2)
      ELSE IF (cname.eq.param(7)) THEN
         READ(line,*) p2
         input(7)=1
         GO TO 7
c ..... Number of filters in the photometric catalogue
      ELSE IF (cname.eq.param(8)) THEN
         READ(line,*) nfilt
         input(8)=1
         GO TO 7
      ELSE 
         WRITE(*,'(A,1x,A)') 'Unknown parameter (ignored)',cname
         GO TO 7
      END IF
 10   CONTINUE
      CLOSE(77)
      IF (input(3).eq.0.and.input(4).eq.0) THEN
         WRITE(*,'(A)') ' Missed both QSO_FILE and STAR_FILE'
         WRITE(*,'(A)') 
     &        ' Output file will be nearly identical to GALAXY_FILE'
         WRITE(*,'(A)') ' Continue anyway? [ y / n ]'
         READ(*,'(A)') cc
         IF (cc.ne.'y'.and.cc.ne.'Y') STOP
      ELSE IF (input(3).eq.0.and.input(4).eq.1) THEN
         WRITE(*,'(A)') ' Missed QSO_FILE'
         WRITE(*,'(A)') 
     &        ' Output file will classify only galaxies/stars'
         WRITE(*,'(A)') ' Continue? [ y / n ]'
         READ(*,'(A)') cc
         IF (cc.ne.'y'.and.cc.ne.'Y') STOP
      ELSE IF (input(3).eq.1.and.input(4).eq.0) THEN
         WRITE(*,'(A)') ' Missed STAR_FILE'
         WRITE(*,'(A)') 
     &        ' Output file will classify only galaxies/QSOs'
         WRITE(*,'(A)') ' Continue? [ y / n ]'
         READ(*,'(A)') cc
         IF (cc.ne.'y'.and.cc.ne.'Y') STOP
      END IF
      DO i=1,nparam
         IF (input(i).eq.0.and.i.ne.3.and.i.ne.4) THEN
            WRITE(*,*) 'Missed parameter ', param(i)
            STOP
         END IF
      END DO
c
      OPEN(10,FILE=filecat,STATUS='old')
c ..... Reading the header (thanks to V. Le Brun)
      WRITE(*,'(A)') ' Reading the photometric catalogue...' 
      header = .FALSE.
      READ(10,'(A)') header_line
      IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
         header= .TRUE.
         i=1
 44      READ(10,'(A)') header_line
         IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
            i=i+1
            GO TO 44
         ELSE
            n_head=i
            j=1
            GO TO 45
         END IF
      END IF
      j=1
 45   READ(10,'(A)',ERR=46,END=46) header_line
      j=j+1
      GO TO 45
 46   nobj1=j
      CLOSE(10)
      WRITE(*,'(A80)') ' File '//filecat
      WRITE(*,'(A,I4,A,I8,A)') ' contains ',n_head,' header lines and ',
     &     nobj1,' objects' 

c Galaxy file
      OPEN(11,FILE=filegal,STATUS='old')
      WRITE(*,'(A)') ' Reading the galaxy photometric redshift file...' 
      j=0
 50   READ(11,'(A)',ERR=51,END=51) linez
      j=j+1
      GO TO 50
 51   nobj2=j
      CLOSE(11)
      WRITE(*,'(A80)') ' File '//filegal
      WRITE(*,'(A,I8,A)') ' contains ',nobj2,' objects' 
      IF (nobj2.ne.nobj1) THEN
         WRITE(*,'(A)') 'Catalogue file and zphot galaxy file contain '
     &        //'different number of objects'
         STOP
      END IF

c QSO file
      IF (input(3).eq.1) THEN
         OPEN(12,FILE=fileqso,STATUS='old')
         WRITE(*,'(A)') ' Reading the QSO photometric redshift file...' 
         j=0
 60      READ(12,'(A)',ERR=61,END=61) linez
         j=j+1
         GO TO 60
 61      nobj3=j
         CLOSE(12)
         WRITE(*,'(A80)') ' File '//fileqso
         WRITE(*,'(A,I8,A)') ' contains ',nobj3,' objects' 
         IF (nobj3.ne.nobj1) THEN
            WRITE(*,'(A)') 'Catalogue file and zphot QSO file '
     &           //'contain different number of objects'
            STOP
         END IF
      END IF

c Star file
      IF (input(4).eq.1) THEN
         OPEN(13,FILE=filestar,STATUS='old')
         WRITE(*,'(A)') ' Reading the star photometric redshift file...' 
         j=0
 70      READ(13,'(A)',ERR=71,END=71) linez
         j=j+1
         GO TO 70
 71      nobj4=j
         CLOSE(13)
         WRITE(*,'(A80)') ' File '//filestar
         WRITE(*,'(A,I8,A)') ' contains ',nobj4,' objects' 
         IF (nobj4.ne.nobj1) THEN
            WRITE(*,'(A)') 'Catalogue file and zphot star file '
     &           //'contain different number of objects'
            STOP
         END IF
      END IF
c
      WRITE(*,'(A)') ' Classifying objects...' 
      OPEN(10,FILE=filecat,STATUS='old')
      IF (header) THEN
         DO i=1,n_head
            READ(10,'(A)') header_line
         END DO
      END IF

      OPEN(11,FILE=filegal,STATUS='old')
      IF (input(3).eq.1) OPEN(12,FILE=fileqso,STATUS='old')
      IF (input(4).eq.1) OPEN(13,FILE=filestar,STATUS='old')
      OPEN(14,FILE=fileout,STATUS='unknown')

      n_tg=0
      n_ts=0
      n_tq=0
      zqso=99.
      zstar=99.
      pqso=-1.
      pstar=-1.
      DO i=1,nobj1
         READ(10,*) id,(amag(k),k=1,nfilt),(amerr(k),k=1,nfilt)
         nobs=0
         DO k=1,nfilt
            IF (amag(k).lt.90..and.amag(k).gt.0..and.amerr(k).lt.90.
     &           .and.amerr(k).gt.0.) nobs=nobs+1
         END DO
         READ(11,'(A)') linez
         last=INDEX(linez,'                   ')
         READ(linez,*) id,zgal,c2gal,pgal
         IF (input(3).eq.1) READ(12,*) id,zqso,c2qso,pqso
         IF (input(4).eq.1) READ(13,*) id,zstar,c2star,pstar
         IF (pgal.lt.p1) n_tg=0
         IF (pgal.ge.p1.and.pgal.lt.p2) n_tg=1
         IF (pgal.ge.p2) n_tg=2
         IF (pqso.lt.p1) n_tq=0
         IF (pqso.ge.p1.and.pqso.lt.p2) n_tq=1
         IF (pqso.ge.p2) n_tq=2
         IF (pstar.lt.p1) n_ts=0
         IF (pstar.ge.p1.and.pstar.lt.p2) n_ts=1
         IF (pstar.ge.p2) n_ts=2
         WRITE(14,1000) linez(1:last),nobs,pstar,zqso,pqso,
     &        n_tg,n_ts,n_tq
      END DO
      CLOSE(10)
      CLOSE(11)
      CLOSE(12)
      CLOSE(13)
      CLOSE(14)
 1000 FORMAT(A,1X,I2,1X,F6.2,1X,F7.4,1X,F6.2,3(2X,I1))
      STOP
      END
