c compute magnitudes in a given filter set from the best fit template 
c at a fixed (spectroscopic) redshift AND THE STELLAR MASS
c N.B. only BC models are supported. 
c in the spectra.param file the tau[Gyr]  must be specified for each SED:
c write 0. for SSP and a number <0 for constant star formation
c possible reading of files .4color: they must be located in same directory 
c of spectra. in *.4color files the interesting columns are 
c 1.log-age-yr, 7.M*, 8.Mgas, 9.Mgalaxy, 10.SFR/yr
c we are interested in M*, because we take into account the return fraction
c input: photometric catalogue with zspec
c output file format: id,zs,chi2,P,itype,iage,age,av,norm,mags,m_abs,starmass
c modif July 18, 2005: photometric error: minimum or added in quadrature
c modif July 22, 2005: limiting magnitude: strict (does it work?) 
c modif July 25, 2005: updated algorithm to compute absolute magnitude
c modif July 26, 2005: corrected choice of ages min and max when AGE_TYPE=0,2
c                      added choice on rebinning
c modif June 15, 2006: corrected choice of M* (220 ages in *.4color files 
c                      instead of 221 present in *.ised)
c modif July 05, 2006: added estimate of the mass error and changed output 
c                      format (ID,z,chi2,P(chi2),itype,iage,age,A_V,norm,
c                      Mgal,M*,mags,m_abs,M*_inf(99%),M*_sup(99%),
c                      M*_inf(90%),M*_sup(90%),M*_inf(68%),M*_sup(68%),sfr,tau
c modif July 06, 2006: added parameter PRIOR to impose some prior following  
c                      Fontana et al. 2004 to avoid old ellipticals with 
c                      huge amount of dust and other unplausible models
c modif July 10, 2006: added parameter SN_LIMIT to check the S/N in each 
c                      filter; if S/N<SN_LIMIT then mag=99 (non detection)
c modif October 10, 2006: added output column with the number of filters 
c                      used in the fit. 
c modif February 10, 2007: changed conversion magAB-> fluxes used in the fit
c                      and used consistent conversion in input and output 
c modif October 02, 2007: changed frequency integrals (too much c/nu^2 factors)
c modif March 02, 2010: corrected of a small bug in agemin when the prior
c                      is switched on and galaxies have z~1. 
c modif April 07, 2010: revised input/output units 
c                     OUTPUT_TYPE=0 [erg/cm2/s/A]
c                     OUTPUT_TYPE=1 [muJy]
c                     OUTPUT_TYPE=2 [mag AB]
c                     checked for input in AB magnitudes (MAG_TYPE=1)
c
c    Version: 07/04/2010
      PROGRAM hyper_zspec_mass
      IMPLICIT NONE
      INTEGER i,ii,iii,j,k,kk,l,ks,n_files,icomp,ncomp,kfil,iw_vega,iw,
     .     n_filter,iob,nob,id,iout_s,iout_m,law_rd,law_err,nfil,iroot,
     .     matrix,iw_ob,i_typ,i_red,i_age,i_lyf,nz,itype,iage,nlyf,
     .     npoiz,n_sol,iwav,npoiz1,npoiz2,last,nred,ired,law_nd,i_lyman,
     .     izsol,mxage,mxtyp,mxfil,mxred,mxlyf,mxwl,ndeg,magtyp,iblank,
     .     input,iunit_sed,nparam,ipass,izmin1,iagemax,iagemin,iageminp,
     .     iread,jread,nf,np,mxft,mxwf,nfilt_mabs,n_fil_w,iztyp,
     .     i_sol,iout,iz,iforbid,n_head,npercent0,npercent,istep,nsteps,
     .     ncolzs,nagelog,iext,imabs,iage_check,ir_filters,icheck,
     .     irebin,imod,nmod,isol,iprior
      REAL*8 zs,w_vega,f_vega,t_bc,scl,z1,z2,amag,amag1,err_p,
     .     mag_nd,err_nd,err_min,avmin,avmax,averr,wl,surf,c_corr,
     .     p,err_fl,dz,zmax,zmin,zz,y,zop,filter,av,age,fl,qq,qqq,
     .     w0,wmin1,wmax1,wmin0,wmax0,wsyn,ysyn,yob,vega,flux,mag_nd1,
     .     conv_mag,bp1,time_tot,part_time,f_jy,err_jy,deg,pchi2,
     .     qchi2,zero,chi2,r_chi2,amabs_1,amabs_2,amagb,dif_w,
     .     dif_w_min,fl_temp,mag_temp,func,prob,err,x1,x2,ww,h0,
     .     om,ov,ageuni,tz,r,zeropoint,color,chi2sol,qq2,fl_nd,
     .     fl1,rlin,amagb_tmp,chi2min,c,fnu_vega,rnu_vega,wrnu,
     .     rrnu,f1,f2,ffnu,trapz1,fact_lyf,r_lyf,frac,wlmax,wlmin,
     .     flux0,amagfit,fluxfit,yy,rcol(100),factmass,pi4,tau,
     .     starmass,galmass,sfrgal,d_l,dl,starmass_tmp,galmass_tmp,
     .     sfrgal_tmp,rnorm,tau_tmp,z_form,age_form,age_min,age_max,
     .     wl_ir,dmod,chi2array,massarray,sm1,sm2,chi2arr_tmp,val,
     .     massarr_tmp,age_z,age_z1,avlim,ttaulim,zform1,sn,sn_lim,
     .     rrnu2,yynu,wwnu,fluxn,fluxerrn,flnu,fluxfitn,rrnuf,wrnuf,
     .     flnu_norm
      REAL*8 fff,bm,wt,sxx,sxy,b,bb,bbb
      REAL wv,fv,ff,w_bc,f_bc,t_bc0,cputime,rmag,rerr_p,aa,rzs,agelog,
     .     amstar,amgas,amgal,sfr,agelog_tmp,amstar_tmp,amgas_tmp,
     .     amgal_tmp,sfr_tmp
c
      PARAMETER (err=1.D-8,nparam=35,pi4=3.141592654D0*4.D0,nmod=250000)
      PARAMETER (mxage=225)
      INCLUDE 'dimensionmass.dec'
      INCLUDE 'filter.dec'

      LOGICAL header
      CHARACTER header_line*5000
      CHARACTER namech(mxtyp)*160,file4color(mxtyp)*160,
     .     sed_abc(mxtyp)*2,sed_tau(mxtyp)*12,line*240
      CHARACTER*160 spectra,spectra0,spectra1,spectra2,filterout,
     .     amatrix,filedef,filein,filt_file,cat_in,spec_vega,
     .     filtresfile,line_sed,copyright1,copyright2
      CHARACTER clog*1,ccat*1,cz*1,csed1*1,csed2*1,cage*1,cout_m*1,
     .     cout_s*1,root*12,date*24,vv*5,cprior*1
      CHARACTER*250 fmt,fmt1,fmt2,fmt3,fmt4
      CHARACTER*40 cname,param(nparam)
c
      DIMENSION input(nparam),kfil(10)
      DIMENSION ks(mxtyp),iw(mxtyp),tau(mxtyp),t_bc0(mxage),t_bc(mxage)
      DIMENSION w_bc(mxwl,mxtyp),f_bc(mxwl,mxage,mxtyp),ff(mxwl)
      DIMENSION w_vega(4000),f_vega(4000),vega(0:mxfil)
      DIMENSION flux0(0:mxfil),f2(0:mxfil)
      DIMENSION c_corr(mxfil),fl_nd(mxfil),fl1(mxfil)
      DIMENSION f_jy(mxfil),err_jy(mxfil),fl_temp(mxfil),mag_temp(mxfil)
      DIMENSION rmag(mxfil),rerr_p(mxfil),amagfit(mxfil)
      DIMENSION fluxfit(mxfil),fluxfitn(mxfil)
      DIMENSION amag(mxfil),amag1(mxfil),err_p(mxfil)
      DIMENSION mag_nd(mxfil),mag_nd1(mxfil),err_nd(mxfil),law_nd(mxfil)
      DIMENSION ysyn(mxwl),wsyn(mxwl),ww(mxwl),yob(mxwl),y(mxwl)
      DIMENSION wl(0:mxfil),surf(0:mxfil),conv_mag(0:mxfil)
      DIMENSION flux(0:mxfil,mxage,mxtyp,mxred),yy(mxwl)
      DIMENSION fluxn(0:mxfil),fluxerrn(0:mxfil),yynu(mxwl),wwnu(mxwl)
      DIMENSION zeropoint(0:mxfil),nfil(0:mxfil),i_lyman(mxfil)
      DIMENSION fnu_vega(4000),rnu_vega(4000)
      DIMENSION wrnu(mxwf),rrnu(mxwf),rrnu2(mxwf),ffnu(mxwf)
      DIMENSION wrnuf(mxwf,0:mxfil),rrnuf(mxwf,0:mxfil),flnu(mxwf)
      DIMENSION val(3),p(3),sm1(3),sm2(3)
      DIMENSION chi2array(nmod),massarray(nmod),chi2arr_tmp(nmod),
     .     massarr_tmp(nmod)
      DIMENSION agelog_tmp(mxtyp,mxage),amstar_tmp(mxtyp,mxage),
     .     amgas_tmp(mxtyp,mxage),amgal_tmp(mxtyp,mxage),
     .     sfr_tmp(mxtyp,mxage)
      DIMENSION agelog(mxtyp,mxage),amstar(mxtyp,mxage),
     .     amgas(mxtyp,mxage),amgal(mxtyp,mxage),sfr(mxtyp,mxage)
c
      EXTERNAL cputime,rlin,filter,pchi2,qchi2,zero,func,tz,trapz1
      EXTERNAL color,d_l
      COMMON/et2/fl(mxfil),err_fl(mxfil)
      COMMON/compose/icomp
      COMMON/nfree/deg,prob
      COMMON/filterres/filtresfile
      COMMON/lyf_sigma/fact_lyf
c_________________________________________________________________________

      vv='2.3'
      copyright1='    Hyperzmass code - Version '//vv
     .     //' (April 07, 2010)' 
      copyright2=' by Micol Bolzonella'
      WRITE(*,*)
      WRITE(*,'(a80)') copyright1
      WRITE(*,*)
      WRITE(*,'(a80)') copyright2
      WRITE(*,*)

      jread=0                   ! Control on reading A0VSED
      iread=0                   ! Control on reading Filter Library
      c=2.9979D+18              ! speed of light in AA/s
      factmass=pi4*3.085678D0*3.085678D48/3.826D33
      nlyf=1                    ! considering only the mean Madau value
      r_lyf=1.D0
      fact_lyf=1.D0
      law_err=0
c      flnu_norm=1.E-30

      ndeg=1
      deg=DBLE(REAL(ndeg))

c ..... Probabilities for the confidence intervals
      p(1)=0.99D0
      p(2)=0.90D0
      p(3)=0.68D0
      x1=0.D0
      x2=200.D0
      DO i=1,3
         prob=p(i)
         val(i)=zero(func,x1,x2,err) ! deltachi2 for P=p(i) and 1 degree of freedom
      END DO

c....... limits in absolute magnitude

      amabs_1=-100.0D0
      amabs_2=100.0D0
      sn_lim=-1.D0
      irebin=0
      imabs=1

 11   filein='hyperzmass.param'
      last=INDEX(filein,'  ')-1
      WRITE(*,*) 'Name of parameter file'
      WRITE(*,*) '[default = ',filein(1:last),']'
      READ(*,'(A)') line
      IF (line.ne.' ') THEN
         READ(line,'(A)',ERR=11) filein
      END IF
      time_tot=DBLE(cputime())
c ------------------------------------------------------------------------
c                       Reading parameters file 
c ------------------------------------------------------------------------

c ..... Parameters in common hyperz & make_catalog
      param(1)='A0VSED'
      param(2)='FILTERS_RES'
      param(3)='FILTERS_FILE'
      param(4)='TEMPLATES_FILE'
      param(5)='MAG_TYPE'
      param(6)='ERR_MAG_MIN'
      param(7)='ERR_MIN_LAW'
      param(8)='REDDENING_LAW'
      param(9)='AV_MIN'
      param(10)='AV_MAX'
      param(11)='AV_STEP'
      param(12)='OUTPUT_FILE'
      param(13)='H0'
      param(14)='OMEGA_M'
      param(15)='OMEGA_V'
      param(16)='FILT_M_ABS'
      param(17)='CATALOG_FILE'
c      param(18)='AGE_CHECK' ! obsolete
      param(18)='AGE_TYPE'
      param(19)='M_ABS_MIN'
      param(20)='M_ABS_MAX'
      param(21)='NCOL_ZSPEC'
      param(22)='SED_OBS_FILE'
      param(23)='SED_TEMP_FILE'
      param(24)='MATRIX'
      param(25)='SPECTRUM'
      param(26)='OUTPUT_TYPE'
      param(27)='M_ABS_ALG'
      param(28)='WL_MAX'
      param(29)='IR_FILTERS'
      param(30)='Z_FORM'
      param(31)='AGE_MIN'
      param(32)='AGE_MAX'
      param(33)='AGE_REBIN'
      param(34)='PRIOR'
      param(35)='SN_LIMIT'

      DO i=1,nparam
         input(i)=0
      END DO

      OPEN(77,FILE=filein,STATUS='old',ERR=11)
      WRITE(*,*) 'Reading parameter file :'

 7    READ(77,'(A)',ERR=10,END=10) line
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            GO TO 8
         END IF
      END DO
      GO TO 7  ! blank line
 8    IF (line(1:1).eq.'#') GO TO 7 ! comment line
      iblank=INDEX(line,' ')
      cname=line(1:iblank-1) ! variable name
      line=line(iblank:)
      DO i=1,80
         IF (line(i:i).ne.' ') THEN
            line=line(i:)
            IF (line(1:1).eq.'#') THEN
               WRITE(*,*) 'Missed value of variable ', cname
               STOP
            END IF
            GO TO 9
         END IF
      END DO
      WRITE(*,*) 'Missed value of variable ', cname
      STOP
 9    iblank=INDEX(line,' ')
      line=line(1:iblank-1) ! part of string containing value of variable
c Parameters in common hyperz & make_catalog
c ..... File containing Vega spectrum
      IF (cname.eq.param(1)) THEN
         READ(line,'(A)') spec_vega
         input(1)=1
         GO TO 7
c ..... File FILTER.RES
      ELSE IF (cname.eq.param(2)) THEN
         READ(line,'(A)') filtresfile
         input(2)=1
         GO TO 7
c ..... File containing filter parameters and instructions for 
c       non detected objects (nd) or out of file (oof) for each filter
c       0 -> flux = -1 (filter not considered) for either nd or oof
c       1 -> flux = -1 for oof, flux = 0, err_flux = flux_nd for nd
c       2 -> flux = -1 for oof, flux = flux_nd/2, err_flux = flux_nd/2 for nd
c       3 -> flux = -1 for oof, mag = mag_nd,  err = err_nd for nd
c       4 -> flux = -1 for oof, reject solutions with flux > flux_nd
      ELSE IF (cname.eq.param(3)) THEN
         READ(line,'(A)') filt_file
         input(3)=1
         GO TO 7
c ..... File containing synthetic spectra filenames
      ELSE IF (cname.eq.param(4)) THEN
         READ(line,'(A)') filedef
         input(4)=1
         GO TO 7
c ..... Magnitude type:
c       0 -> standard magnitudes
c       1 -> AB magnitudes
c       2 -> Flux in microJansky
      ELSE IF (cname.eq.param(5)) THEN
         READ(line,*) magtyp
         IF (magtyp.ne.0.and.magtyp.ne.1.and.magtyp.ne.2) THEN
            WRITE(*,*) 'MAG_TYPE must be 0 , 1 or 2'
            STOP
         END IF
         input(5)=1
         GO TO 7
c ..... Minimum error in magnitude (if error < err_min => err = err_min)
      ELSE IF (cname.eq.param(6)) THEN
         READ(line,*) err_min
         input(6)=1
         GO TO 7
c ..... Photometric errors:
c       0 -> if error < err_min => err = err_min
c       1 -> err_min added in quadrature to err in catalogue
      ELSE IF (cname.eq.param(7)) THEN
         READ(line,*) law_err
         IF (law_err.ne.0.and.law_err.ne.1) THEN
            WRITE(*,*) 'ERR_MIN_LAW must be 0 or 1'
            STOP
         END IF
         input(7)=1
         GO TO 7
c ..... Reddening:
c       0 -> No reddening
c       1 -> Allen, 1976, standard MW
c       2 -> Seaton, 1979, standard MW
c       3 -> Fitzpatrick, 1986, LMC
c       4 -> Prevot, 1984 + Bouchet, 1985, SMC
c       5 -> Calzetti et al., 2000
      ELSE IF (cname.eq.param(8)) THEN
         READ(line,*) law_rd
         IF (law_rd.ne.0.and.law_rd.ne.1.and.law_rd.ne.2.and.
     .        law_rd.ne.3.and.law_rd.ne.4.and.law_rd.ne.5) THEN
            WRITE(*,*) 'REDDENING_LAW must be 0 -- 5'
            STOP
         END IF
         input(8)=1
         GO TO 7
c ..... Av_max, Av_min
      ELSE IF (cname.eq.param(9)) THEN
         READ(line,*) avmin
         input(9)=1
         GO TO 7
      ELSE IF (cname.eq.param(10)) THEN
         READ(line,*) avmax
         input(10)=1
         GO TO 7
c ..... Reddening step used to build hypercube
      ELSE IF (cname.eq.param(11)) THEN
         READ(line,*) averr
         input(11)=1
         GO TO 7
c ..... Output file
      ELSE IF (cname.eq.param(12)) THEN
         READ(line,'(A)') spectra
         iblank=INDEX(spectra,' ')
         spectra0=spectra(1:iblank-1)//'.zmass'
         spectra1=spectra(1:iblank-1)//'.zmobs_sed'
         spectra2=spectra(1:iblank-1)//'.zmtemp_sed'
         filterout=spectra(1:iblank-1)//'.out_zmass'
         input(12)=1
         GO TO 7
c ..... Cosmological parameters to compute the universe age and M_abs 
      ELSE IF (cname.eq.param(13)) THEN
         READ(line,*) h0
         input(13)=1
         GO TO 7
      ELSE IF (cname.eq.param(14)) THEN
         READ(line,*) om
         input(14)=1
         GO TO 7
      ELSE IF (cname.eq.param(15)) THEN
         READ(line,*) ov
         input(15)=1
         GO TO 7
c Parameters relative to hyperz only
c ..... Filter for the absolute magnitude
      ELSE IF (cname.eq.param(16)) THEN
         READ(line,*) nfilt_mabs
         input(16)=1
         GO TO 7
c ..... File containing photometric catalogue
      ELSE IF (cname.eq.param(17)) THEN
         READ(line,'(A)') cat_in
         input(17)=1
         GO TO 7
c ..... Check the age of the object < universe age (y/n)
      ELSE IF (cname.eq.'AGE_CHECK') THEN
         WRITE(*,*) 'AGE_CHECK is obsolete, but it is OK for this time'
         cage=line(1:1)
         IF (cage.eq.'y'.or.cage.eq.'Y') THEN
            iage_check=1
         ELSE
            iage_check=0
         END IF
         input(18)=1
         GO TO 7
c ..... Constraint for age (it will supersede AGE_CHECK)
c       0. free
c       1. free with age <  age universe-age(T_FORM)
c       2. fixed age: age=age of universe-T(Z_FORM)
      ELSE IF (cname.eq.param(18)) THEN
         READ(line,*) iage_check
         IF (iage_check.ne.0.and.iage_check.ne.1.and.
     &        iage_check.ne.2) THEN
            WRITE(*,*) 'AGE_TYPE must be 0, 1 or 2'
            STOP
         END IF
         input(18)=1
         GO TO 7
c ..... Limits for absolute magnitude 
      ELSE IF (cname.eq.param(19)) THEN
         READ(line,*) amabs_1
         input(19)=1
         GO TO 7
      ELSE IF (cname.eq.param(20)) THEN
         READ(line,*) amabs_2
         input(20)=1
         GO TO 7
c ..... Number of the col. after the photometric cat. corresponding to zspec
      ELSE IF (cname.eq.param(21)) THEN
         READ(line,*) ncolzs
         input(21)=1
         GO TO 7
c ..... File .zmobs_sed (y/n)
      ELSE IF (cname.eq.param(22)) THEN
         csed1=line(1:1)
         input(22)=1
         GO TO 7
c ..... File .zmtemp_sed (y/n)
      ELSE IF (cname.eq.param(23)) THEN
         csed2=line(1:1)
         input(23)=1
         GO TO 7
c ..... Matrix/obj
      ELSE IF (cname.eq.param(24)) THEN
         cout_m=line(1:1)
         input(24)=1
         GO TO 7
c ..... Spectrum/obj
      ELSE IF (cname.eq.param(25)) THEN
         cout_s=line(1:1)
         input(25)=1
         GO TO 7
c ..... Output SED: 
c       0 -> erg/s/cm^2/A ; 1 -> microJansky ; 2 -> AB magnitudes
      ELSE IF (cname.eq.param(26)) THEN
         READ(line,*) iunit_sed
         IF (iunit_sed.ne.0.and.iunit_sed.ne.1.and.iunit_sed.ne.2) THEN
            WRITE(*,*) 'OUTPUT_TYPE must be 0, 1 or 2'
            STOP
         END IF
         input(26)=1
         GO TO 7
c ..... Algorithm to compute absolute magnitudes: 
c       0. nearest filter 1. best fit normalization
      ELSE IF (cname.eq.param(27)) THEN
         READ(line,*) imabs 
         IF (imabs.ne.0.and.imabs.ne.1) THEN
            WRITE(*,*) 'MAG_ABS_ALG must be 0 or 1'
            STOP
         END IF
         input(27)=1
         GO TO 7
c ..... Maximum rest frame wavelength [Angstrom] to be uses, according to 
c       parameter IR_FILTERS
      ELSE IF (cname.eq.param(28)) THEN
         READ(line,*) wl_ir
         input(28)=1
         GO TO 7
c ..... Possibilities using IR_FILTERS
c       0. use all available filters
c       1. use filters with lambda_eff > WL_MAX tot only when 
c          lambda_eff/(1+z) < WL_MAX
c       2. do not use filters with lambda_eff > WL_MAX
      ELSE IF (cname.eq.param(29)) THEN
         READ(line,*) ir_filters
         IF (ir_filters.ne.0.and.ir_filters.ne.1
     &        .and.ir_filters.ne.2) THEN
            WRITE(*,*) 'IR_FILTERS must be 0, 1 or 2'
            STOP
         END IF
         input(29)=1
         GO TO 7
c ..... Formation redshift (used only if AGE_TYPE=1 or 2)
      ELSE IF (cname.eq.param(30)) THEN
         READ(line,*) z_form
         input(30)=1
         GO TO 7
c ..... Minimum age to be considered
      ELSE IF (cname.eq.param(31)) THEN
         READ(line,*) age_min
         input(31)=1
         GO TO 7
c ..... Maximum age to be considered
      ELSE IF (cname.eq.param(32)) THEN
         READ(line,*) age_max
         input(32)=1
         GO TO 7
c ..... Age rebinning: 
c       0. no (221 ages) 1. yes (51 ages)
      ELSE IF (cname.eq.param(33)) THEN
         READ(line,*) irebin 
         IF (irebin.ne.0.and.irebin.ne.1) THEN
            WRITE(*,*) 'AGE_REBIN must be 0 or 1'
            STOP
         END IF
         input(33)=1
         GO TO 7
c ..... Apply prior tu exclude unplausible models (y/n)
      ELSE IF (cname.eq.param(34)) THEN
         cprior=line(1:1)
         input(34)=1
         GO TO 7
c ..... Signal-to-noise ratio limit for detection/non detection
      ELSE IF (cname.eq.param(35)) THEN
         READ(line,*) sn_lim
         input(35)=1
         GO TO 7
      ELSE 
         WRITE(*,'(a,1x,a)') 'Unknown parameter (ignored)',cname
         GO TO 7
      END IF

 10   CONTINUE
      CLOSE(77)
      iout_m=1
      iout_s=1
      IF (cout_m.eq.'y'.or.cout_m.eq.'Y') iout_m=2
      IF (cout_s.eq.'y'.or.cout_s.eq.'Y') iout_s=2

      IF (law_rd.eq.0) THEN
         avmin=0.0D0
         avmax=0.0D0
         averr=1.0D0
      END IF
      nred=NINT(ABS(avmax-avmin)/averr)+1
      IF (ABS(avmin+averr*(nred-1)-avmax).gt.1.D-6) nred=nred-1
      IF (nred.gt.mxred) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of reddening steps exceeding the maximum [max=',
     .        mxred,']'
         STOP
      END IF

      age_form=tz(h0,om,ov,z_form) ! age of universe at z_form
c      write(*,*) age_form, tz(h0,om,ov,0.D0)

      OPEN(10,FILE=filedef,STATUS='old',ERR=78)
      iii=1
 2230 READ(10,'(A)',ERR=2220,END=2220) line_sed
      IF (line_sed(1:1).eq.'#'.or.line_sed.eq.' ') GO TO 2230
      DO i=1,160
         IF (line_sed(i:i).ne.' ') THEN
            line_sed=line_sed(i:)
            last=INDEX(line_sed,' ')-1
            namech(iii)=line_sed(1:last)
            GO TO 2221
         END IF
      END DO
      WRITE(*,*) 'Missed name SED files '
      STOP
 2221 line_sed=line_sed(last+1:)
      DO i=1,160
         IF (line_sed(i:i).ne.' ') THEN
            line_sed=line_sed(i:)
            sed_abc(iii)=line_sed(1:2)
            IF (sed_abc(iii).ne.'BC') THEN
               last=INDEX(namech(iii),' ')
               WRITE(*,'(a)') 'The type of SED '//namech(iii)(1:last)//
     .              ' is '//sed_abc(iii)
               WRITE(*,'(a)') 'Only BC type is supported'
               STOP
            END IF
            last=INDEX(line_sed,' ')-1
            GO TO 2222
         END IF
      END DO
      WRITE(*,*) 'Missed type of SED file ', namech(iii)
      STOP
 2222 line_sed=line_sed(last+1:)
c      read(line_sed,*) tau(iii),zsun(iii)
      DO i=1,160
         IF (line_sed(i:i).ne.' ') THEN
            line_sed=line_sed(i:)
            last=INDEX(line_sed,' ')-1
            sed_tau(iii)=line_sed(1:last)
            GO TO 2223
         END IF
      END DO
      WRITE(*,*) 'Missed tau of SFR for file', namech(iii)
      STOP
 2223 read(sed_tau(iii),*) tau(iii)
      iii=iii+1
      GO TO 2230
 2220 n_files=iii-1
      IF (n_files.gt.mxtyp) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of files exceeding the maximum [max=',mxtyp,']'
         STOP
      END IF
      WRITE(*,*) 'There are ',n_files,' synthetic spectra'
      CLOSE(10)

      CALL filter0
      OPEN(10,FILE=filt_file,STATUS='old',ERR=80)
      i=0
 134  CONTINUE
      READ(10,'(A)',ERR=135,END=135) line
      IF (line(1:1).eq.'#'.or.line.eq.' ') GO TO 134
      i=i+1
      ncomp=1
      DO k=1,80
         IF (line(k:k).eq.'&') THEN
            line(k:k)=' '
            ncomp=ncomp+1
         ELSE IF (line(k:k).eq.'#') THEN
            GO TO 136
         END IF
      END DO
 136  CONTINUE
      IF (ncomp.eq.1) THEN 
         READ(line,*,ERR=82) nfil(i),c_corr(i),law_nd(i),
     .        mag_nd(i),err_nd(i)
         IF (law_nd(i).ne.0.and.law_nd(i).ne.1.and.law_nd(i).ne.2.and.
     .        law_nd(i).ne.3.and.law_nd(i).ne.4) THEN
            WRITE(*,*) 'Non detection law must be 0, 1, 2, 3 or 4'
            WRITE(*,*) 'Check filter ', nfil(i)
            STOP
         END IF
         IF (nfil(i).gt.icomp) THEN
            WRITE(*,1998) nfil(i),icomp
            STOP
         ELSE
c ..... simple filter
            kfil(1)=nfil(i)
            CALL compos_new(1,kfil,wl(i),surf(i),bp1)
            GO TO 134
         END IF
      ELSE 
         READ(line,*,ERR=82) (kfil(j),j=1,ncomp),c_corr(i),law_nd(i),
     .                 mag_nd(i),err_nd(i)
         IF (law_nd(i).ne.0.and.law_nd(i).ne.1.and.law_nd(i).ne.2.and.
     .        law_nd(i).ne.3.and.law_nd(i).ne.4) THEN
            WRITE(*,*) 'Non detection law must be 0, 1, 2, 3 or 4'
            WRITE(*,*) 'Check filter ', nfil(i)
            STOP
         END IF
c ..... composite filter
         icomp=icomp+1
         nfil(i)=icomp
         CALL compos_new(ncomp,kfil,wl(i),surf(i),bp1)
         GO TO 134
      END IF
 1998 FORMAT('Filter number ',I4, ' out of range!',/,
     .       'Maximum = ',I4)

 135  n_filter=i
      IF (n_filter.gt.mxfil) THEN
         WRITE(*,'(1x,a,i3,a)') 
     .        'Number of filters exceeding the maximum [max=',mxfil,']'
         STOP
      END IF
      WRITE(*,*) 'There are ',n_filter,' filters'
      nfil(0)=nfilt_mabs ! filter for absolute magnitude estimates
      IF (nfil(0).gt.icomp) THEN
         WRITE(*,1998) nfil(0),icomp
         STOP
      END IF
      kfil(1)=nfil(0)
      CALL compos_new(1,kfil,wl(0),surf(0),bp1)
      CLOSE(10)

      OPEN(10,FILE=cat_in,STATUS='old',ERR=79)

c ..... Reading the header (thanks to V. Le Brun)
      header = .FALSE.
      READ(10,'(A)',ERR=79) header_line
      IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
         header= .TRUE.
         i=1
 44      READ(10,'(A)',ERR=79) header_line
         IF (header_line(1:1).eq.'#'.or.header_line.eq.' ') THEN
            i=i+1
            GO TO 44
         ELSE
            n_head=i
         END IF
      END IF
      CLOSE(10)
      OPEN(10,FILE=cat_in,STATUS='old',ERR=79)
c ..... Skipping the header (thanks to V. Le Brun)
      IF (header) THEN
         DO i=1,n_head
            READ(10,'(A)') header_line
         END DO
      END IF

      iob=0
 1340 CONTINUE
      iob=iob+1
      READ(10,*,ERR=1350,END=1350) id,(rmag(i),i=1,n_filter),
     .     (rerr_p(i),i=1,n_filter),(rcol(i),i=1,ncolzs)
      rzs=rcol(ncolzs)
      GO TO 1340
 1350 nob=iob-1
      WRITE(*,*) 'There are ',nob,' objects in the catalogue'
      CLOSE(10)

c -----------------------------------------------------------------------
c                       Reading synthetic spectra 
c -----------------------------------------------------------------------

c ..... Vega

      WRITE (*,'(1x,a)') 'Reading reference sed:'
      OPEN(10,FILE=spec_vega,STATUS='old')
      READ(10,'(A)') line
      WRITE(*,'(1x,a)') line (1:78)
c      scl=1.D0
      scl=2.D-17
      iii=1
 223  READ(10,*,ERR=222,END=222) wv,fv
      w_vega(iii)=DBLE(wv)
      f_vega(iii)=DBLE(fv)*scl
      iii=iii+1
      GO TO 223
 222  iw_vega=iii-1
      CLOSE(10)
      WRITE(*,*) 'There are ',iw_vega,' points in Vega'
      jread=iw_vega

c ..... SED
      WRITE(*,*) 'Reading synthetic spectra :'
      DO i=1,n_files
         last=INDEX(namech(i),' ')
         WRITE(*,2005) i,namech(i)(1:last)
c BC templates only
         OPEN(1,FILE=namech(i),STATUS='old',
     .        FORM='unformatted',ERR=25)
         READ (1) ks(i),(t_bc0(ii),ii=1,ks(i)) ! ks = n. temporal intervals
         READ (1) iw(i),(w_bc(ii,i),ii=1,iw(i)) !Number of wavelengths
         IF (iw(i).gt.mxwl) THEN
            WRITE(*,'(a,i6)') 'Number of wavelengths in GISSEL '
     .           //'templates exceeding the maximum. Max = ',mxwl
            WRITE(*,'(a)') 
     .           ' Reduce GISSEL wavelenghts or modify mxwl in '
     .           //'dimensionmass.dec and recompile'
            STOP
         END IF
         k=0
         DO j=1,ks(i)
            READ (1) iw(i),(ff(ii),ii=1,iw(i))
            IF ((irebin.eq.1.and.(t_bc0(j)/1.E9.le.0.01.and.
     &           (FLOAT(j)/10.-FLOAT(j/10)).lt.1.E-6)
     &           .or.(t_bc0(j)/1.E9.gt.0.01.and.
     &           t_bc0(j)/1.E9.le.3..and.
     &           (FLOAT(j-49)/3.-FLOAT((j-49)/3)).lt.1.E-6)
     &           .or.(t_bc0(j)/1.E9.gt.3..and.
     &           t_bc0(j)/1.E9.le.20..and.
     &           (FLOAT(j-15)/4.-FLOAT((j-15)/4)).lt.1.E-6)).or.
     &           irebin.eq.0) THEN
               k=k+1
               t_bc(k)=DBLE(t_bc0(j))
               DO ii=1,iw(i)
                  IF (w_bc(ii,i).lt.912.) THEN
                     f_bc(ii,k,i)=0.D0
                  ELSE
                     f_bc(ii,k,i)=DBLE(ff(ii))
                  END IF
               END DO
            END IF
         END DO
         ks(i)=k
         CLOSE(1)
      END DO
 2005 FORMAT(' Type ', I3, ' : ', A)

      WRITE(*,*) 'Reading *.4color files :'
      DO i=1,n_files
         iext=INDEX(namech(i),'.ised')-1
         file4color(i)=namech(i)(1:iext)//'.4color'
         WRITE(*,'(a)') file4color(i)(1:iext+7)
c impose values for record age=0 present in ised files and not in 4color
         j=1
         agelog(i,j)=-99.
         amstar(i,j)=1.
         amgas(i,j)=0.
         amgal(i,j)=1.
         sfr(i,j)=0.
         OPEN(20,FILE=file4color(i),STATUS='old',ERR=26)
 2224    READ(20,'(a)',ERR=2225,END=2225) line
         IF(line.eq.' '.or.line(1:1).eq.'#') GO TO 2224
         j=j+1
         READ(line,*) agelog(i,j),aa,aa,aa,aa,aa,amstar(i,j),
     .        amgas(i,j),amgal(i,j),sfr(i,j)
         GO TO 2224
 2225    CLOSE(20)
         nagelog=j
         IF (irebin.eq.1) THEN
            k=0
            DO j=1,nagelog
               agelog_tmp(i,j)=agelog(i,j)
               amstar_tmp(i,j)=amstar(i,j)
               amgas_tmp(i,j)=amgas(i,j)
               amgal_tmp(i,j)=amgal(i,j)
               sfr_tmp(i,j)=sfr(i,j)
               agelog(i,j)=0.
               amstar(i,j)=0.
               amgas(i,j)=0.
               amgal(i,j)=0.
               sfr(i,j)=0.
               IF ((t_bc0(j)/1.E9.le.0.01.and.
     &              (FLOAT(j)/10.-FLOAT(j/10)).lt.1.E-6)
     &              .or.(t_bc0(j)/1.E9.gt.0.01.and.
     &              t_bc0(j)/1.E9.le.3..and.
     &              (FLOAT(j-49)/3.-FLOAT((j-49)/3)).lt.1.E-6)
     &              .or.(t_bc0(j)/1.E9.gt.3..and.
     &              t_bc0(j)/1.E9.le.20..and.
     &              (FLOAT(j-15)/4.-FLOAT((j-15)/4)).lt.1.E-6)) THEN
                  k=k+1
                  agelog(i,k)=agelog_tmp(i,j)
                  amstar(i,k)=amstar_tmp(i,j)
                  amgas(i,k)=amgas_tmp(i,j)
                  amgal(i,k)=amgal_tmp(i,j)
                  sfr(i,k)=sfr_tmp(i,j)
               END IF
            END DO
         END IF
      END DO

c -----------------------------------------------------------------------
c                    Filters and .out_phot file
c -----------------------------------------------------------------------

      OPEN(18,file=filterout,status='unknown')
      CALL fdate(date)
c      CALL fdate_(date)   ! IBM
      WRITE(18,'(a)') '# Date: '//date
      WRITE(18,'(a)') '# Photometric catalogue : '
      last=INDEX(cat_in,' ')-1
      WRITE(18,'(2x,a)')  cat_in(1:last)
      WRITE(18,'(a)') 
     .     '# Templates SEDs:'
      DO itype=1,n_files
         last=INDEX(namech(itype),' ')-1
         WRITE(18,'(2x,i2,2x,a,2x,a,2x,f9.4)') itype,
     .     namech(itype)(1:last),sed_abc(itype),tau(itype)
      END DO
      WRITE(18,'(a)') '# Characteristics of filters'
      WRITE(18,'(a)') '#  n     wl_eff     surface   conv_AB'
     .     //'  m_corr mlim(Vega) nd'
      zz=0.D0

c ..... Conversion from AB to standard magnitudes: conv_mag = AB - Vega
c compute f_Vega(nu)
      DO i=1,iw_vega
         fnu_vega(i)=f_vega(i)*w_vega(i)*w_vega(i)/c
         rnu_vega(i)=c/w_vega(i)
      END DO

      DO k=0,n_filter
         vega(k)=filter(nfil(k),w_vega,f_vega,iw_vega,zz)
         zeropoint(k)=-color(nfil(k),nfilt_mabs,
     .                       w_vega,f_vega,iw_vega,zz)

c transmission function R(nu). 
c Change of variable: lambda -> nu : dlambda = dnu * c/nu^2
         DO i=1,np(nfil(k))
            wrnu(i)=c/r(i,nfil(k),1) ! frequency
            rrnu(i)=r(i,nfil(k),2) ! R(nu)=R(lambda)
            wrnuf(i,k)=wrnu(i) ! frequency of filter k
            rrnuf(i,k)=rrnu(i) 
            rrnu2(i)=rrnu(i)*c/(wrnu(i)*wrnu(i))! R(nu)*c/nu^2
         END DO
c R(nu)*f_vega(nu)
         DO i=1,np(nfil(k))
            l=0
            ffnu(i)=rrnu(i)*rlin(wrnu(i),rnu_vega,fnu_vega,iw_vega,l)
         END DO
c integrals with trapezoids
         f1=trapz1(wrnu,ffnu,np(nfil(k)))
         f2(k)=trapz1(wrnu,rrnu,np(nfil(k)))
c         f2b(k)=trapz1(wrnu,rrnu2,np(nfil(k))) ! int(R(nu)*c/nu^2)=surf(k)
         conv_mag(k)=-2.5D0*DLOG10(f1/f2(k))-48.60D0
c ..... Wavelength limit for the break in each filter 
c         aw_limit(k)=r(np(nfil(k))-1,nfil(k),1)
      END DO
      DO k=1,n_filter
c ..... Flux corresponding to the limiting magnitude
         IF (magtyp.eq.0) THEN
            mag_nd1(k)=mag_nd(k)
            fl_nd(k)=vega(k)*
     .           10**(-0.4D0*(mag_nd(k)+c_corr(k)))/surf(k) 
            mag_nd(k)=-2.5D0*DLOG10(fl_nd(k)*surf(k)/vega(k)) !Vega lim mag
         ELSE IF (magtyp.eq.1) THEN
            mag_nd1(k)=mag_nd(k)
            fl_nd(k)=10**(-0.4D0*(mag_nd1(k)+c_corr(k)+48.6D0))
            mag_nd(k)=mag_nd1(k)-conv_mag(k) !Vega limiting mag
c            fl_nd(k)=vega(k)*
c     .           10**(-0.4D0*(mag_nd1-conv_mag(k)+c_corr(k)))/surf(k)
c            fl_nd(k)=(10**(-0.4D0*(mag_nd(k)+c_corr(k)+48.6D0))*
c     .           (c/(wl(k)**2))) 
c            mag_nd(k)=-2.5D0*DLOG10(fl_nd(k)*surf(k)/vega(k)) !Vega lim mag
         ELSE IF (magtyp.eq.2) THEN 
            fl_nd(k)=mag_nd(k)*1.D-12*0.5D0*(c/(wl(k)**2)) 
            mag_nd(k)=-2.5D0*DLOG10(fl_nd(k)*surf(k)/vega(k)) !Vega lim mag
         END IF
         WRITE(18,1999) nfil(k),wl(k),surf(k),conv_mag(k),
     .        c_corr(k),mag_nd(k),law_nd(k)
      END DO
 1999 FORMAT(2X,I3,1X,F11.3,1X,F10.3,1X,F7.3,1X,F6.3,1X,F7.3,
     .     4X,I2)
      WRITE(18,'(a)') '#    AVmin     AVmax    AVstep  red_law'
      WRITE(18,'(4X,3(F5.2,5X),I2)') avmin,avmax,averr,law_rd
      WRITE(18,'(a)') 
     .     '#    Lyman_forest [factor(s) multiplying <tau_eff>]'
      IF (nlyf.eq.1) WRITE(18,'(4X,F6.3)') r_lyf
      IF (nlyf.eq.3) WRITE(18,'(4X,3(F6.3,2X))') 
     .     1.D0/r_lyf,r_lyf/r_lyf,r_lyf
c      WRITE(18,'(a)') '#    Mmin      Mmax     Mfilter  mag type'
      WRITE(18,'(a)') 
     .     '#    Mmin     Mmax     Mfilter input_mag_type err_mag_min'
      IF (magtyp.eq.0) WRITE(18,'(4X,2(F7.2,1X),6X,I3,6X,A5,1X,F7.3)') 
     .     amabs_1,amabs_2,nfilt_mabs,'Vega',err_min
      IF (magtyp.eq.1) WRITE(18,'(4X,2(F7.2,1X),6X,I3,6X,A5,1X,F7.3)') 
     .     amabs_1,amabs_2,nfilt_mabs,'AB',err_min
      IF (magtyp.eq.2) WRITE(18,'(4X,2(F7.2,1X),6X,I3,6X,A5,1X,F7.3)') 
     .     amabs_1,amabs_2,nfilt_mabs,'muJy',err_min
      WRITE(18,'(a)') '#    Age_type age_min[Gyr] age_max[Gyr]  z_form'
      WRITE(18,'(6X,I3,6X,F7.3,6X,F7.3,6X,F6.2)') 
     .     iage_check,age_min,age_max,z_form
      IF (cprior.eq.'y'.or.cprior.eq.'Y') THEN
         WRITE(18,'(a)') '#    Prior: YES'
      ELSE
         WRITE(18,'(a)') '#    Prior: NO'
      END IF
      WRITE(18,'(a)') '#    Omega_0   Omega_v  H_0'
      WRITE(18,'(4X,3(F5.2,5X))') om,ov,h0
c -----------------------------------------------------------------------
c                          Loop over the objects 
c -----------------------------------------------------------------------
      WRITE(*,'(1x,a)') 'I''m working ...'
      OPEN(13,FILE=spectra0,STATUS='unknown',ERR=81)
      IF (csed1.eq.'y'.or.csed1.eq.'Y')
     .     OPEN(14,FILE=spectra1,STATUS='unknown',ERR=81)
      IF (csed2.eq.'y'.or.csed2.eq.'Y')
     .     OPEN(15,FILE=spectra2,STATUS='unknown',ERR=81)
      WRITE(fmt,2001) n_filter
      WRITE(fmt1,2006) n_filter,n_filter
      WRITE(fmt2,2007) n_filter,n_filter
      WRITE(fmt3,2008) n_filter
      WRITE(fmt4,2009) n_filter
 2001 FORMAT ('(1x,i12,2x,f9.5,1x,f8.3,1x,f6.2,1x,i3,1x,i3,1x,
     &     f8.4,1x,f5.2,1x,e10.4,2(1x,f12.6),',i2,'(1x,F8.4),
     &     1x,f9.4,6(1x,f12.6),1x,e12.5,1x,f9.4,1x,i3)')
 2006 FORMAT('(1x,i12,',i2,'(1x,F8.4),',i2,'(1x,F8.4))')
 2007 FORMAT('(1x,i12,',i2,'(1x,E12.5),',i2,'(1x,E12.5))')
 2008 FORMAT('(1x,i12,',i2,'(1x,F8.4))')
 2009 FORMAT('(1x,i12,',i2,'(1x,E12.5))')

      OPEN(16,FILE=cat_in,STATUS='old',ERR=79)
c ..... Skipping the header (thanks to V. Le Brun)
      IF (header) THEN
         DO i=1,n_head
            READ(16,'(A)') header_line
         END DO
      END IF

      iob=0
13400 CONTINUE
      iob=iob+1
      READ(16,*,ERR=13500,END=13500) id,(rmag(i),i=1,n_filter),
     .     (rerr_p(i),i=1,n_filter),(rcol(i),i=1,ncolzs)
      rzs=rcol(ncolzs)
      zs=DBLE(rzs)
      IF (zs.ge.0.D0.and.zs.lt.9.D0) THEN
         DO iii=1,n_filter
            IF (((magtyp.eq.0.or.magtyp.eq.1).and.
     .           (rmag(iii).lt.90.0.and.rmag(iii).gt.1.)).or.
     .           (magtyp.eq.2.and.rmag(iii).ge.0.)) GO TO 13401
         END DO
         WRITE(*,*) 'WARNING: non detected object id ', id
      END IF
      r_chi2=99.D0
      qq=0.D0
      i_typ=0
      i_age=0
      age=99.D0
      av=0.D0
      bb=0.D0
      amagb=99.D0
      galmass=-100.D0
      starmass=-100.D0
      iw_ob=0
      DO i=1,3
         sm1(i)=-100.D0
         sm2(i)=-100.D0
      END DO
      sfrgal=-100.D0
      tau_tmp=-100.D0
      DO k=1,n_filter
         amagfit(k)=99.D0
      END DO
      WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,galmass,
     .     starmass,(amagfit(k),k=1,n_filter),amagb,sm1(1),sm2(1),
     .     sm1(2),sm2(2),sm1(3),sm2(3),sfrgal,tau_tmp,iw_ob
c ..... File .zmobs_sed
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
         DO i=1,n_filter
            fl1(i)=-1.D0
            err_fl(i)=0.D0
            amag(i)=99.D0
            err_p(i)=99.D0
         END DO
         IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
            WRITE(14,fmt2) id,(fl1(i),i=1,n_filter),
     .           (err_fl(i),i=1,n_filter)
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(14,fmt1) id,(amag(i),i=1,n_filter),
     .           (err_p(i),i=1,n_filter)
         END IF
      END IF
c ..... File .zmtemp_sed
      IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
         DO k=1,n_filter
            mag_temp(k)=99.D0
            fl_temp(k)=-1.D0
         END DO
         IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
            WRITE(15,fmt4) id,(fl_temp(k),k=1,n_filter)
         ELSE IF (iunit_sed.eq.2) THEN
            WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filter)
         END IF
      END IF
      GO TO 13400
         
c ..... Computing the fluxes and errors in each filter
13401 iw_ob=0 
      DO iii=1,n_filter
         amag(iii)=DBLE(rmag(iii))
         err_p(iii)=DBLE(rerr_p(iii))
         i_lyman(iii)=0
c ..... check the S/N in each filter if mag is detected
         IF (amag(iii).gt.0.D0.and.amag(iii).lt.90.D0
     &        .and.err_p(iii).lt.90.D0) THEN 
            IF (magtyp.eq.0.or.magtyp.eq.1) THEN
               IF (err_p(iii).lt.err_min) THEN ! avoid to divide by zero
                  sn=(2.5D0/DLOG(10.D0))/err_min
               ELSE
                  sn=(2.5D0/DLOG(10.D0))/err_p(iii)
               END IF
            ELSE
               IF (err_p(iii).lt.err_min) THEN ! avoid to divide by zero
                  sn=amag(iii)/err_min
               ELSE
                  sn=amag(iii)/err_p(iii)
               END IF
            END IF
            IF (sn.lt.sn_lim) THEN ! set mag and dmag as upper limit
               IF (magtyp.eq.0.or.magtyp.eq.1) THEN
c                  amag(iii)=99.D0
                  err_p(iii)=99.D0
               ELSE
                  amag(iii)=-99.D0
                  err_p(iii)=-99.D0
               END IF
            END IF
         END IF
c photometric errors
         IF (law_err.eq.0) THEN
            IF (err_p(iii).lt.err_min) err_p(iii)=err_min
         ELSE
c added in quadrature
            err_p(iii)=SQRT(err_min**2+err_p(iii)**2)
         END IF
c ..... check if the filter is an IR filter and if it is usable
c       0. use all available filters
c       1. use filters with lambda_eff > 20000A tot only when 
c          lambda_eff/(1+z) < 20000A
c       2. do not use filters with lambda_eff > 20000A
         IF (wl(iii).le.wl_ir.or.ir_filters.eq.0) THEN 
c optical filter, no need for check, or no check on IR filters requested
            GO TO 13402
         END IF
         IF (ir_filters.eq.1) THEN 
            IF (wl(iii)/(1.D0+zs).le.wl_ir) THEN ! ok use the IR filter
               GO TO 13402
            ELSE 
c do not use it: put the magnitude as not observed and decrease the 
c degrees of freedom
               amag(iii)=-1.D0
               err_p(iii)=-1.D0
            END IF
         ELSE IF (ir_filters.eq.2) THEN ! do not use all IR filters
            amag(iii)=-1.D0
            err_p(iii)=-1.D0
         END IF
13402    CONTINUE
c ..... Detected object in filter iii - Vega or AB magnitudes
         IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .        (amag(iii).gt.1.D0.and.amag(iii).lt.90.D0).and.
     .        err_p(iii).lt.90.D0) THEN
            iw_ob=iw_ob+1
            IF (magtyp.eq.0) fl(iii)=vega(iii)*
     .           10**(-0.4D0*(amag(iii)+c_corr(iii)))/surf(iii) 
            IF (magtyp.eq.1) fl(iii)=10**(-0.4D0*(amag(iii)+
     &           c_corr(iii)+48.6D0))
c            IF (magtyp.eq.1) fl(iii)=(10**(-0.4D0*(amag(iii)+
c     .           c_corr(iii)+48.6D0)))*f2(iii)/surf(iii)
c            IF (magtyp.eq.1) fl(iii)=(10**(-0.4D0*(amag(iii)+
c     .           c_corr(iii)+48.6D0))*(c/(wl(iii)**2)))
c            IF (magtyp.eq.1) fl(iii)=vega(iii)*
c     .           10**(-0.4D0*(amag(iii)-conv_mag(iii)+c_corr(iii)))
c     &           /surf(iii)
            err_fl(iii)=0.4D0*DLOG(10.D0)*fl(iii)*err_p(iii)
c amag -> Vega magnitude
            IF (magtyp.eq.0) THEN
               amag1(iii)=amag(iii)
            ELSE IF (magtyp.eq.1) THEN
               amag1(iii)=amag(iii)-conv_mag(iii)
            END IF

c ..... Detected object in filter iii - flux in microJansky
        ELSE IF (magtyp.eq.2.and.amag(iii).ge.0.D0) THEN ! input in muJy
            iw_ob=iw_ob+1
            f_jy(iii)=amag(iii)
            err_jy(iii)=err_p(iii)
            fl(iii)=amag(iii)*1.D-12*0.5D0*(c/(wl(iii)**2))
            err_fl(iii)=err_p(iii)*1.D-12*0.5D0*
     .           (c/(wl(iii)**2))
c amag: flux[muJy] -> Vega magnitude
            amag1(iii)=-2.5D0*DLOG10(fl(iii)*surf(iii)/vega(iii)) 

c ..... Detected object in filter iii as upper limit - Vega or AB magnitudes
c       use non detection law 2
         ELSE IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .           (amag(iii).gt.1.D0.and.amag(iii).lt.90.D0).and.
     .           err_p(iii).gt.90.D0) THEN
            iw_ob=iw_ob+1
            IF (magtyp.eq.0) fl(iii)=vega(iii)*
     .           10**(-0.4D0*(amag(iii)+c_corr(iii)))/surf(iii) 
            IF (magtyp.eq.1) fl(iii)=10**(-0.4D0*(amag(iii)+
     &           c_corr(iii)+48.6D0))
            fl(iii)=0.5D0*fl(iii)
            err_fl(iii)=fl(iii)
            i_lyman(iii)=1
c amag -> Vega magnitude
            IF (magtyp.eq.0) THEN
               amag1(iii)=amag(iii)
            ELSE IF (magtyp.eq.1) THEN
               amag1(iii)=amag(iii)-conv_mag(iii)
            END IF

c ..... Detected object in filter iii as upper limit - flux in microJansky
c       use non detection law 2
         ELSE IF (magtyp.eq.2.and.amag(iii).ge.0..and.
     .           err_p(iii).lt.0.) THEN 
            iw_ob=iw_ob+1
            fl(iii)=amag(iii)*1.D-12*0.5D0*(c/(wl(iii)**2))
            amag1(iii)=-2.5D0*DLOG10(fl(iii)*surf(iii)/vega(iii)) 
            fl(iii)=0.5D0*fl(iii)
            err_fl(iii)=fl(iii)
            f_jy(iii)=0.5D0*amag(iii)
            err_jy(iii)=f_jy(iii)
            i_lyman(iii)=1

c ..... Non observed object in filter iii - Vega or AB magnitudes
         ELSE IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .           amag(iii).le.1.D0) THEN ! out of file objects
            fl(iii)=-1.0D0
            err_fl(iii)=0.0D0
            amag1(iii)=-1.0D0

c ..... Non observed object in filter iii - flux in microJansky
         ELSE IF (magtyp.eq.2.and.ABS(amag(iii)+1.D0).lt.1.D-6) THEN 
            fl(iii)=-1.0D0
            err_fl(iii)=0.0D0
            amag1(iii)=-1.0D0
            f_jy(iii)=-1.0D0
            err_jy(iii)=0.0D0

c ..... Non detected object in filter iii - Vega or AB magnitudes or flux 
c       use selected non detection law and input limiting magnitude or flux
         ELSE IF (((magtyp.eq.0.or.magtyp.eq.1).and.
     .           amag(iii).ge.90.D0).or.
     .           (magtyp.eq.2.and.amag(iii).lt.-90.D0)) THEN 
c ..... Non detection laws: 
c       0. the filter will not be used
c       1. the flux in the filter is set = 0 with 
c          photometric error = limiting flux 
c       2. the flux in the filter and the photometric error are set =
c          0.5*limiting flux
c       3. the flux in the filter is = limiting flux and the photometric
c          error is computed from column 5 of the parameter file
c       4. the filter will not be used in chi2, but solutions with 
c          flux > limiting flux will be rejected
            IF (law_nd(iii).eq.0) THEN
               fl(iii)=-1.0D0
               err_fl(iii)=0.D0
               amag1(iii)=99.D0
               f_jy(iii)=-1.0D0
               err_jy(iii)=0.D0
            ELSE IF (law_nd(iii).eq.1) THEN
               fl(iii)=0.D0
               err_fl(iii)=fl_nd(iii)
               amag1(iii)=99.D0
               i_lyman(iii)=2
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.2) THEN
               fl(iii)=0.5D0*fl_nd(iii)
               err_fl(iii)=fl(iii)
               amag1(iii)=99.D0
               i_lyman(iii)=1
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.3) THEN
               fl(iii)=fl_nd(iii)
               err_p(iii)=err_nd(iii)
               amag1(iii)=99.D0
               IF (magtyp.eq.0.or.magtyp.eq.1) THEN
                  err_fl(iii)=0.4D0*DLOG(10.D0)*fl(iii)*err_p(iii)
               ELSE
                  err_fl(iii)=err_nd(iii)*1.D-12*0.5D0*
     .                 (c/(wl(iii)**2))
               END IF
               i_lyman(iii)=1
               iw_ob=iw_ob+1
            ELSE IF (law_nd(iii).eq.4) THEN
               fl(iii)=-1.D0
               err_fl(iii)=0.D0
               amag1(iii)=99.D0
               i_lyman(iii)=2
            END IF
         END IF
      END DO

      ndeg=iw_ob-1  ! degrees of freedom
      deg=DBLE(REAL(ndeg))

      IF (ndeg.lt.1) THEN
         WRITE(*,*) 'WARNING: impossible to compute object ', iob
c ..... Write outputs in any case 
         r_chi2=99.D0
         qq=0.D0
         i_typ=0
         i_age=0
         age=99.D0
         av=0.D0
         bb=0.D0
         amagb=99.D0
         galmass=-100.D0
         starmass=-100.D0
         DO i=1,3
            sm1(i)=-100.D0
            sm2(i)=-100.D0
         END DO
         sfrgal=-100.D0
         tau_tmp=-100.D0
         DO k=1,n_filter
            amagfit(k)=99.
         END DO
         WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .        galmass,starmass,(amagfit(k),k=1,n_filter),amagb,
     .        sm1(1),sm2(1),sm1(2),sm2(2),sm1(3),sm2(3),sfrgal,
     .        tau_tmp,iw_ob
c ..... File .zmobs_sed
         IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
            DO i=1,n_filter
               fl1(i)=-1.D0
               err_fl(i)=0.D0
               amag(i)=99.D0
               err_p(i)=99.D0
            END DO
            IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
               WRITE(14,fmt2) id,(fl1(i),i=1,n_filter),
     .              (err_fl(i),i=1,n_filter)
            ELSE IF (iunit_sed.eq.2) THEN
               WRITE(14,fmt1) id,(amag(i),i=1,n_filter),
     .              (err_p(i),i=1,n_filter)
            END IF
         END IF
c ..... File .zmtemp_sed
         IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
            DO k=1,n_filter
               mag_temp(k)=99.D0
               fl_temp(k)=-1.D0
            END DO
            IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
               WRITE(15,fmt4) id,(fl_temp(k),k=1,n_filter)
            ELSE IF (iunit_sed.eq.2) THEN
               WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filter)
            END IF
         END IF
         GO TO 13400
      END IF

c -----------------------------------------------------------------------
c                         Computation 
c -----------------------------------------------------------------------
c ..... Open file for the matrix of results (if case)

      matrix=0
      root=' '
      WRITE(root,'(i12)') id
      DO k=1,12
         IF (root(k:k).ne.' ') THEN
            root=root(k:)
            GO TO 22
         END IF
      END DO
 22   iroot=INDEX(root,' ')-1
      IF (iout_m.gt.1) THEN
         amatrix=root(1:iroot)//'.zm'
         matrix=1
         OPEN(11,FILE=amatrix,STATUS='unknown')
      END IF

      DO k=1,n_filter
         amagfit(k)=99.D0
      END DO
      dif_w_min=1.D12
      chi2min=1.D12
      n_fil_w=1
c search for nearest filter at zs 
      DO k=1,n_filter
         dif_w=ABS(wl(k)-wl(0)*(zs+1.D0))
         IF ((magtyp.eq.0.or.magtyp.eq.1).and.
     .        amag(k).gt.1.D0.and.amag(k).lt.90.D0
     .        .and.i_lyman(k).lt.1) THEN
            IF (dif_w.lt.dif_w_min) THEN   
               dif_w_min=dif_w 
               n_fil_w=k  
            END IF
         ELSE IF (magtyp.eq.2.and.amag(k).ge.0.D0) THEN
            IF (dif_w.lt.dif_w_min) THEN   
               dif_w_min=dif_w 
               n_fil_w=k  
            END IF
         END IF
      END DO
      i_typ=1
      i_age=2
      i_red=1
c      IF (cage.eq.'y'.or.cage.eq.'Y') THEN
      IF (iage_check.eq.1.or.iage_check.eq.2) THEN
         ageuni=tz(h0,om,ov,zs)
      END IF

ccccccccccccccccccc begin PRIOR ccccccccccccccccccccc
c some parameter useful only for the prior activated
      avlim=0.6D0
      ttaulim=4.D0
      zform1=1.0D0
      age_z1=tz(h0,om,ov,zform1) ! age of universe at z=zform1
ccccccccccccccccccc end PRIOR ccccccccccccccccccccc
      age_z=tz(h0,om,ov,zs) ! age of universe at zs
      dl=d_l(h0,om,ov,zs)   !luminosity distance in Mpc
      IF (dl.gt.0.D0) dmod=5.D0*DLOG10(dl)+25.D0
c minimum dmod: corresponding to d(~M31)=1000 Kpc
      IF (dmod.lt.25.D0) dmod=25.D0 
c start the counter for number of models
      imod=0
c -----------------------------------------------------------------------
      DO itype=1,n_files        !Scan types
         iagemin=2
         iagemax=ks(itype)
c if age_min and age_max are not in the range of values considered by BC SEDs
c and iage_check=0 then iagemin=2  and iagemax=ks(itype)
         DO iage=2,ks(itype)
            IF (age_min.gt.(t_bc(iage-1)*1.D-9).and.
     .           age_min.le.(t_bc(iage)*1.D-9)) iagemin=iage
            IF (age_max.gt.(t_bc(iage-1)*1.D-9).and.
     .           age_max.le.(t_bc(iage)*1.D-9)) iagemax=iage
         END DO
c free age between age_min and age(z)
         IF (iage_check.eq.1) THEN 
            DO iage=2,ks(itype)
               IF (ageuni-age_form.gt.(t_bc(iage-1)*1.D-9).and.
     .              ageuni-age_form.le.(t_bc(iage)*1.D-9).and.
c choose the minimum between age(z) and age_max
     .              ageuni-age_form.le.t_bc(iagemax)*1.D-9) iagemax=iage
            END DO
c only the two ages around the age(z)
         ELSE IF (iage_check.eq.2) THEN  
            iagemin=2
            iagemax=3
            DO iage=3,ks(itype)
               IF (ageuni-age_form.gt.(t_bc(iage-1)*1.D-9).and.
     .              ageuni-age_form.le.(t_bc(iage)*1.D-9)) THEN
                  iagemin=iage-1
                  iagemax=iage
               END IF
            END DO
         END IF
ccccccccccccccc begin PRIOR cccccccccccccccccccccccc
c skip models with tau < 0.6Gyr and ages for which zform1<1 
c *** only for objects with zs<1 ***
         IF ((cprior.eq.'y'.or.cprior.eq.'Y').and.
     &        zs.lt.1.D0.and.tau(itype)-0.6D0.le.err.and.
     &        tau(itype).gt.err) THEN
c find the minimum age to have age(zs)-t_bc < age(z=1)
            DO iage=2,ks(itype)
               IF (age_z-t_bc(iage)*1.D-9.ge.age_z1) THEN
c the last valid iage corresponds to the minumum age
                  iageminp=iage
               END IF
            END DO
            IF (iageminp.gt.iagemin) THEN 
               iagemin=iageminp
            END IF
c            write(*,*) itype,tau(itype),iagemin,age_z1,
c     &           age_z-t_bc(iagemin)*1.D-9
         END IF
ccccccccccccccc end PRIOR cccccccccccccccccccccccc
c -----------------------------------------------------------------------
         DO iage=iagemin,iagemax ! Scan ages
            DO k=1,iw(itype)
               y(k)=DBLE(f_bc(k,iage,itype))
               ww(k)=DBLE(w_bc(k,itype))
            END DO
            age=t_bc(iage)*1.D-9
c -----------------------------------------------------------------------
            DO ired=1,nred      ! Scan reddening
c -----------------------------------------------------------------------
               av=avmin+(ired-1)*averr
ccccccccccccccc begin PRIOR cccccccccccccccccccccccc
c skip models with AV > 0.6 and no star formation, i.e. age/tau>4 
               IF ((cprior.eq.'y'.or.cprior.eq.'Y').and.
     &              av-avlim.gt.err) THEN
c check if the galaxy is star forming
                  IF (age/tau(itype)-ttaulim.ge.0.D0) THEN ! not star forming 
c just skip this model (possible problems when reading matrix file?)
                     GO TO 333
                  END IF
               END IF
ccccccccccccccc end PRIOR cccccccccccccccccccccccc
               DO k=1,iw(itype)
                  yy(k)=y(k)
               END DO
               IF (av.gt.0.0) THEN
                  CALL red_spec(ww,yy,av,yob,1,iw(itype),law_rd) 
                  DO k=1,iw(itype)
                     yy(k)=yob(k)
                  END DO
               END IF

c ..... Lyman alpha forest
               CALL lyman_forest(ww,yy,iw(itype),zs)
               IF (magtyp.eq.1) THEN
                  DO k=1,iw(itype)
                     yynu(k)=yy(k)*ww(k)*ww(k)/c ! f(nu) of template
                     wwnu(k)=c/ww(k) ! frequency of template
                  END DO
               END IF
c ..... Compute fluxes in broad band filters
               DO k=0,n_filter
                  flux(k,iage,itype,ired)=
     .                 filter(nfil(k),ww,yy,iw(itype),zs)/
     .                 surf(k)
                  flux0(k)=filter(nfil(k),ww,yy,iw(itype),0.D0)/
     .                 surf(k)
                  IF (magtyp.eq.1) THEN
                     l=0
                     DO kk=1,np(nfil(k))
                        wrnu(kk)=wrnuf(kk,k) ! frequency of R(nu)
                        rrnu(kk)=rrnuf(kk,k) ! R(nu)
                        flnu(kk)=rrnu(kk)*rlin(wrnu(kk)*(1.+zs),wwnu,
     &                       yynu,iw(itype),l)
                     END DO
                     fluxn(k)=trapz1(wrnu,flnu,np(nfil(k)))
     &                    *(1.D0+zs)/f2(k)
                  END IF
               END DO
c ..... Probability for zs, iage, itype, ired
               sxx=0.D0
               sxy=0.D0
               b=0.D0

               DO k=1,n_filter
                  IF (fl(k).ge.1.D-100) THEN ! no oof objects
                     IF (flux(k,iage,itype,ired).lt.1.D-100
     .                    .and.i_lyman(k).eq.1) THEN
                        fl1(k)=0.D0
                     ELSE
                        fl1(k)=fl(k)
                        wt=1.D0/(err_fl(k)**2)
                        IF (magtyp.eq.0) THEN
                           sxx=sxx+
     .                          flux(k,iage,itype,ired)
     .                          *flux(k,iage,itype,ired)
     .                          *wt
                           sxy=sxy+fl1(k)*wt*
     .                          flux(k,iage,itype,ired)
                        ELSE
c                           write(*,*) 'S',k,sxx,sxy,fluxn(k),fl1(k),wt
                           sxx=sxx+fluxn(k)*fluxn(k)*wt
                           sxy=sxy+fl1(k)*wt*fluxn(k)
                        END IF
                     END IF
                  ELSE
                     fl1(k)=-1.D0
                  END IF
               END DO
               b=sxy/sxx
               chi2=0.D0
               DO k=1,n_filter
                  IF (fl1(k).ge.0.D0) THEN ! no oof obj.
                     IF (magtyp.eq.0) THEN
                        chi2=chi2+((fl1(k)
     .                       -b*flux(k,iage,itype,ired))
     .                       /err_fl(k))**2
                     ELSE
                        chi2=chi2+((fl1(k)-b*fluxn(k))/err_fl(k))**2
c                        write(*,*) chi2,b,fl1(k),sxy,sxx
c                     write(72,
c     &        '(i10,1x,i2,1x,i6,1x,i2,1x,i3,1x,i2,8(1x,d11.4))') 
c     &                       id,k,imod,itype,iage,ired,fluxn(k),
c     &                       f2(k),fl1(k),err_fl(k),chi2,b,
c     &                       sxx,sxy
                     END IF
                  END IF
               END DO
c star mass and SFR
               rnorm=b*dl*dl*factmass
               starmass_tmp=rnorm*amstar(itype,iage)
c               galmass_tmp=rnorm*amgal(itype,iage)
c               sfrgal_tmp=rnorm*sfr(itype,iage)
               IF (tau(itype).gt.1.D-8) THEN ! tau models
                  sfrgal_tmp=rnorm*DEXP(-age/tau(itype))/tau(itype)
     &                 *1.D-9
                  galmass_tmp=rnorm*(1.-DEXP(-age/tau(itype)))
               ELSE IF (tau(itype).lt.-1.D-8) THEN ! constant star formation
                  sfrgal_tmp=rnorm
                  galmass_tmp=rnorm*age*1.D9
               ELSE IF (ABS(tau(itype)).lt.1.D-8) THEN ! SSP 
                  sfrgal_tmp=0.D0
                  galmass_tmp=rnorm
               END IF
c update number of models and write arrays with chi2 and mass
               imod=imod+1
               chi2array(imod)=chi2
               massarray(imod)=DLOG10(starmass_tmp)
c write matrix 
               IF (matrix.gt.0) THEN
                  r_chi2=chi2/deg ! reduced chi2
                  qqq=100.D0*qchi2(0.5D0*deg,0.5D0*chi2)
c Absolute magnitude:
c 0. use the nearest observed filter to minimize k-correction (to be checked)
c 1. compute directly from the SED best fit 
                  amagb_tmp=0.0D0
                  IF (imabs.eq.0) THEN ! use the nearest rest frame filter 
                     amagb_tmp=99.
                     IF (flux(n_fil_w,iage,itype,ired).gt.0.D0) THEN
                        amagb_tmp=amag1(n_fil_w)-dmod-
     .                       2.5D0*DLOG10(flux0(0)*surf(0)*vega(n_fil_w)
     .                       /(flux(n_fil_w,iage,itype,ired)*
     .                       surf(n_fil_w)*vega(0)))
                     ELSE 
c use the best fit if it is impossible to compute it from the nearest filter
                        amagb_tmp=-2.5D0*DLOG10(b*flux0(0)*surf(0)/
     .                       vega(0))-dmod
                     END IF
                  ELSE IF (imabs.eq.1) THEN ! use the best fit normalization
                     amagb_tmp=-2.5D0*DLOG10(b*flux0(0)*surf(0)/vega(0))
     .                    -dmod
                  END IF
                  IF (magtyp.eq.1.and.amagb_tmp.lt.90.) 
     .                 amagb_tmp=amagb_tmp+conv_mag(0)
                  WRITE(11,2012) id,zs,itype,iage,age,av,r_chi2,qqq,
     &                 b,amagb_tmp,DLOG10(galmass_tmp),
     &                 DLOG10(starmass_tmp),sfrgal_tmp,tau(itype)
               END IF
c Check limiting magnitude if law_nd=4: 
c reject solution if flux template > limiting flux
               icheck=0
               DO k=1,n_filter
                  IF (fl1(k).lt.0.D0.and.law_nd(k).eq.4.and.
     .                 flux(k,iage,itype,ired).gt.fl_nd(k)) THEN 
                     icheck=icheck+1                     
                  END IF
               END DO
c New minimum chi2
               IF (chi2-chi2min.lt.0.D0.and.icheck.eq.0) THEN 
c Compute absolute magnitude (if not already calculated) 
                  IF (matrix.le.0) THEN
                     amagb_tmp=0.0D0
                     IF (imabs.eq.0) THEN ! use the nearest rest frame filter 
                        amagb_tmp=99.
                        IF (flux(n_fil_w,iage,itype,ired).gt.0.D0) THEN
                           amagb_tmp=amag1(n_fil_w)-dmod-
     .                          2.5D0*DLOG10(flux0(0)*surf(0)*
     .                          vega(n_fil_w)/
     .                          (flux(n_fil_w,iage,itype,ired)*
     .                          surf(n_fil_w)*vega(0)))
                        ELSE 
c use the best fit if it is impossible to compute it from the nearest filter
                           amagb_tmp=-2.5D0*DLOG10(b*flux0(0)*surf(0)/
     .                          vega(0))-dmod
                        END IF
                     ELSE IF (imabs.eq.1) THEN ! use the best fit normalization
                        amagb_tmp=-2.5D0*DLOG10(b*flux0(0)*surf(0)/
     .                       vega(0))-dmod
                     END IF
                     IF (magtyp.eq.1.and.amagb_tmp.lt.90.) 
     .                    amagb_tmp=amagb_tmp+conv_mag(0)
                  END IF
c Check absolute magnitude: if amagb not in [amabs_1,amabs_2] reject solution
                  IF (amagb_tmp.ge.amabs_1.and.
     .                 amagb_tmp.le.amabs_2) THEN
                     chi2min=chi2
                     bb=b
                     i_typ=itype
                     i_age=iage
                     i_red=ired
                     amagb=amagb_tmp
                     starmass=starmass_tmp
                     sfrgal=sfrgal_tmp
                     galmass=galmass_tmp
                  END IF
               END IF
 333           CONTINUE
            END DO
         END DO
      END DO
      CLOSE(11)                 !close output matrix

c -----------------------------------------------------------------------

      r_chi2=chi2min/deg        ! reduced chi2
      qq=100.D0*qchi2(0.5D0*deg,0.5D0*chi2min)
c..... sorting chi2 values with masses
      CALL sort2(imod,massarray,chi2array)
c..... computing error bars
c find the position of the best fit
      DO i=1,imod
         IF (ABS(massarray(i)-DLOG10(starmass)).lt.err.and.
     &        ABS(chi2array(i)-chi2min).lt.err) isol=i
      END DO

c values after starmass
      IF (isol.eq.imod) THEN ! best chi2 at the maximum mass
         DO i=1,3              ! z at P = 99 90 and 68%
            sm2(i)=massarray(isol)
         END DO
      ELSE
         DO i=isol,imod
            chi2arr_tmp(i-isol+1)=chi2array(i)
            massarr_tmp(i-isol+1)=massarray(i)
         END DO
c due to chi2 oscillations, to find confidence intervals find 
c the maximum mass with deltachi2<=val(i)
         DO j=1,3               ! mass at P = 99 90 and 68%
            sm2(j)=-100.D0
            DO i=2,imod-isol+1
               IF (chi2arr_tmp(i)-chi2min.lt.val(j)) THEN
                  sm2(j)=massarr_tmp(i)
               END IF
            END DO
         END DO
c if no points below deltachi2 have been found
c the solution is between best fit and nearest point in mass
c sort in chi2 to find the nearest point in chi2 (avoid too small errors)
         CALL sort2(imod-isol+1,chi2arr_tmp,massarr_tmp)
         DO j=1,3               ! mass at P = 99 90 and 68%
            IF (sm2(j).le.-99.9D0) THEN !
               l=0
               sm2(j)=rlin(val(j)+chi2min,chi2arr_tmp,
     &              massarr_tmp,imod-isol+1,l)
            END IF
            IF (sm2(j).lt.massarray(1))     sm2(j)=massarray(1)
            IF (sm2(j).gt.massarray(imod))  sm2(j)=massarray(imod)
            IF (sm2(j).lt.DLOG10(starmass)) sm2(j)=DLOG10(starmass)
         END DO
      END IF

c values before starmass
      IF (isol.eq.1) THEN ! best chi2 at the minimum mass
         DO i=1,3              ! z at P = 99 90 and 68%
            sm1(i)=massarray(isol)
         END DO
      ELSE
         DO i=1,isol
            chi2arr_tmp(i)=chi2array(isol-i+1)
            massarr_tmp(i)=massarray(isol-i+1)
         END DO
         DO j=1,3               ! mass at P = 99 90 and 68%
            sm1(j)=-100.D0
            DO i=2,isol
               IF (chi2arr_tmp(i)-chi2min.lt.val(j)) THEN
                  sm1(j)=massarr_tmp(i)
               END IF
            END DO
         END DO
c sort in chi2 to find the nearest point in chi2 (avoid too small errors)
         CALL sort2(isol,chi2arr_tmp,massarr_tmp)
         DO j=1,3               ! mass at P = 99 90 and 68%
            IF (sm1(j).le.-99.9D0) THEN
               l=0
               sm1(j)=rlin(val(j)+chi2min,chi2arr_tmp,
     &              massarr_tmp,isol,l)
            END IF
            IF (sm1(j).lt.massarray(1))     sm1(j)=massarray(1)
            IF (sm1(j).gt.massarray(imod))  sm1(j)=massarray(imod)
            IF (sm1(j).gt.DLOG10(starmass)) sm1(j)=DLOG10(starmass)
         END DO
      END IF

c..... File .zmobs_sed
      IF (csed1.eq.'y'.or.csed1.eq.'Y') THEN
         IF (iunit_sed.eq.0) THEN ! [output in erg/cm2/s/A]
            IF (magtyp.eq.1) THEN ! [input in AB mag]
               DO k=1,n_filter
c detected filter and error
                  IF (rmag(k).gt.0..and.rmag(k).lt.90..and.
     &                 rerr_p(k).lt.90.D0) THEN 
c IR filter ok
                     IF (wl(k).le.wl_ir.or.ir_filters.eq.0.or.
     &                    (ir_filters.eq.1.and.wl(k)/(1.D0+zs).le.
     &                    wl_ir)) THEN
                        fluxn(k)=(10**(-0.4D0*(rmag(k)+
     .                       c_corr(k)+48.6D0)))*f2(k)/surf(k)
c IR filter not considered
                     ELSE
                        fluxn(k)=-(10**(-0.4D0*(rmag(k)+
     .                       c_corr(k)+48.6D0)))*f2(k)/surf(k)
                     END IF
                     fluxerrn(k)=0.4D0*DLOG(10.D0)*rerr_p(k)*fluxn(k)
c "variable" upper limit use non detection law=2
                  ELSE IF (rmag(k).gt.0..and.rmag(k).lt.90..and.
     &                    rerr_p(k).ge.90.D0) THEN 
c keep the sign of err_p for IR not used filters
                     fluxn(k)=0.5*(10**(-0.4D0*(rmag(k)+
     &                    c_corr(k)+48.6D0)))*f2(k)/surf(k)
                     fluxn(k)=sign(fluxn(k),err_p(k))
                     fluxerrn(k)=fluxn(k)
c normal non detection
                  ELSE IF (rmag(k).ge.90.) THEN
                     IF (law_nd(k).eq.0) THEN
                        fluxn(k)=-1.D0
                        fluxerrn(k)=0.D0
                     ELSE IF (law_nd(k).eq.1) THEN
                        fluxn(k)=0.D0
                        fluxerrn(k)=(10**(-0.4D0*(mag_nd1(k)+
     .                    c_corr(k)+48.6D0)))*f2(k)/surf(k)
                     ELSE IF (law_nd(k).eq.2) THEN
                        fluxn(k)=0.5*(10**(-0.4D0*(mag_nd1(k)+
     &                    c_corr(k)+48.6D0)))*f2(k)/surf(k)
                        fluxerrn(k)=fluxn(k)
                     ELSE IF (law_nd(k).eq.3) THEN
                        fluxn(k)=(10**(-0.4D0*(mag_nd1(k)+
     &                    c_corr(k)+48.6D0)))*f2(k)/surf(k)
                        fluxerrn(k)=0.4D0*DLOG(10.D0)*fluxn(k)*err_nd(k)
                     ELSE IF (law_nd(k).eq.4) THEN
                        fluxn(k)=(10**(-0.4D0*(mag_nd1(k)+
     .                    c_corr(k)+48.6D0)))*f2(k)/surf(k)
                        fluxerrn(k)=fluxn(k)                 
                     END IF
c not observed
                  ELSE
                     fluxn(k)=-1.D0
                     fluxerrn(k)=0.D0
                  END IF
               END DO
               WRITE(14,fmt2) id,(fluxn(k),k=1,n_filter),
     &              (fluxerrn(k),k=1,n_filter)
            ELSE ! [input in Vega mag; muJy to be checked]
               DO k=1,n_filter
c detected and undetected filters and errors
c IR filter ok
                  IF (wl(k).le.wl_ir.or.ir_filters.eq.0.or.
     &                 (ir_filters.eq.1.and.wl(k)/(1.D0+zs).le.
     &                 wl_ir)) THEN
                     fluxn(k)=fl(k)
c IR filter not considered
                  ELSE
                     fluxn(k)=-fl(k)
                  END IF
                  fluxerrn(k)=err_fl(k)
c non detection + filter not used or not observed
                  IF (magtyp.eq.0) THEN ! [input in Vega mag]
                     IF ((rmag(k).ge.90..and.law_nd(k).eq.0).or.
     &                    rerr_p(k).lt.0.) THEN
                        fluxn(k)=-1.D0
                        fluxerrn(k)=0.D0
                     END IF
                  ELSE IF (magtyp.eq.2) THEN ! [input in muJy]
                     IF ((rmag(k).le.-90..and.law_nd(k).eq.0).or.
     &                    rerr_p(k).lt.0.) THEN
                        fluxn(k)=-1.D0
                        fluxerrn(k)=0.D0
                     END IF
                  END IF
               END DO
               WRITE(14,fmt2) id,(fluxn(k),k=1,n_filter),
     &              (fluxerrn(k),k=1,n_filter)
            END IF
         ELSE IF (iunit_sed.eq.1) THEN ! [output in muJy]
            IF (magtyp.eq.0) THEN ! [input in mag Vega]
               DO k=1,n_filter
c detected filter and error
                  IF (rmag(k).gt.0..and.rmag(k).lt.90..and.
     &                 rerr_p(k).lt.90.D0) THEN 
c IR filter ok
                     IF (wl(k).le.wl_ir.or.ir_filters.eq.0.or.
     &                    (ir_filters.eq.1.and.wl(k)/(1.D0+zs).le.
     &                    wl_ir)) THEN
                        f_jy(k)=fl1(k)*(wl(k)**2/c)*1.D+29

c IR filter not considered
                     ELSE
                        f_jy(k)=-fl1(k)*(wl(k)**2/c)*1.D+29
                     END IF
                     err_jy(k)=0.4D0*DLOG(10.D0)*f_jy(k)*err_p(k)
c "variable" upper limit use non detection law=2
                  ELSE IF (rmag(k).gt.0..and.rmag(k).lt.90..and.
     &                    rerr_p(k).ge.90.D0) THEN 
c keep the sign of err_p for IR not used filters
                     f_jy(k)=0.5*fl1(k)*(wl(k)**2/c)*1.D+29
                     f_jy(k)=sign(f_jy(k),err_p(k))
                     err_jy(k)=f_jy(k)
c normal non detection
                  ELSE IF (rmag(k).ge.90.) THEN
                     IF (law_nd(k).eq.0) THEN
                        f_jy(k)=-1.D0
                        err_jy(k)=0.D0
                     ELSE IF (law_nd(k).eq.1) THEN
                        f_jy(k)=0.D0
                        err_jy(k)=fl_nd(k)*(wl(k)**2/c)*1.D+29
                     ELSE IF (law_nd(k).eq.2) THEN
                        f_jy(k)=0.5*fl_nd(k)*(wl(k)**2/c)*1.D+29
                        err_jy(k)=f_jy(k)
                     ELSE IF (law_nd(k).eq.3) THEN
                        f_jy(k)=fl_nd(k)*(wl(k)**2/c)*1.D+29
                        err_jy(k)=0.4D0*DLOG(10.D0)*f_jy(k)*err_nd(k)
                     ELSE IF (law_nd(k).eq.4) THEN
                        f_jy(k)=fl_nd(k)*(wl(k)**2/c)*1.D+29
                        err_jy(k)=f_jy(k)                 
                     END IF
c not observed
                  ELSE
                     f_jy(k)=-1.D0
                     err_jy(k)=0.D0
                  END IF
               END DO
            ELSE IF (magtyp.eq.1) THEN ! [input in mag AB]
               DO k=1,n_filter
c detected filter and error
                  IF (rmag(k).gt.0..and.rmag(k).lt.90..and.
     &                 rerr_p(k).lt.90.D0) THEN 
c IR filter ok
                     IF (wl(k).le.wl_ir.or.ir_filters.eq.0.or.
     &                    (ir_filters.eq.1.and.wl(k)/(1.D0+zs).le.
     &                    wl_ir)) THEN
                        f_jy(k)=(10**(-0.4D0*(amag(k)+c_corr(k)
     &                       +48.6D0)))*1.D29

c IR filter not considered
                     ELSE
                        f_jy(k)=-(10**(-0.4D0*(amag(k)+c_corr(k)
     &                       +48.6D0)))*1.D29
                     END IF
                     err_jy(k)=0.4D0*DLOG(10.D0)*err_p(k)*f_jy(k)
c "variable" upper limit use non detection law=2
                  ELSE IF (rmag(k).gt.0..and.rmag(k).lt.90..and.
     &                    rerr_p(k).ge.90.D0) THEN 
c keep the sign of err_p for IR not used filters
                     f_jy(k)=0.5*(10**(-0.4D0*(amag(k)+c_corr(k)
     &                       +48.6D0)))*1.D29
                     f_jy(k)=sign(f_jy(k),err_p(k))
                     err_jy(k)=f_jy(k)
c normal non detection
                  ELSE IF (rmag(k).ge.90.) THEN
                     IF (law_nd(k).eq.0) THEN
                        f_jy(k)=-1.D0
                        err_jy(k)=0.D0
                     ELSE IF (law_nd(k).eq.1) THEN
                        f_jy(k)=0.D0
                        err_jy(k)=10**(-0.4D0*(mag_nd1(k)+c_corr(k)
     &                       +48.6D0))*1.D+29
                     ELSE IF (law_nd(k).eq.2) THEN
                        f_jy(k)=0.5*10**(-0.4D0*(mag_nd1(k)+c_corr(k)
     &                       +48.6D0))*1.D+29
                        err_jy(k)=f_jy(k)
                     ELSE IF (law_nd(k).eq.3) THEN
                        f_jy(k)=10**(-0.4D0*(mag_nd1(k)+c_corr(k)
     &                       +48.6D0))*1.D+29
                        err_jy(k)=0.4D0*DLOG(10.D0)*f_jy(k)*err_nd(k)
                     ELSE IF (law_nd(k).eq.4) THEN
                        f_jy(k)=10**(-0.4D0*(mag_nd1(k)+c_corr(k)
     &                       +48.6D0))*1.D+29
                        err_jy(k)=f_jy(k)                 
                     END IF
c not observed
                  ELSE
                     f_jy(k)=-1.D0
                     err_jy(k)=0.D0
                  END IF
               END DO
            ELSE IF (magtyp.eq.2) THEN ! [input in muJy]
c to be updated for upper limits and non detection
               DO k=1,n_filter
                  f_jy(k)=amag(k)
                  err_jy(k)=err_p(k)
               END DO
            END IF
            WRITE(14,fmt2) id,(f_jy(i),i=1,n_filter),
     .           (err_jy(i),i=1,n_filter)
         ELSE IF (iunit_sed.eq.2) THEN ! [output in AB mag]
            IF (magtyp.eq.0.or.magtyp.eq.1) THEN ! [input in mag Vega or AB]
               DO k=1,n_filter
                  IF (amag(k).gt.90.D0) THEN
                     amag(k)=mag_nd(k)+conv_mag(k)
                     err_p(k)=99.D0
                  ELSE IF (amag1(k).lt.1.D0) THEN
                     amag(k)=-1.D0
                     err_p(k)=-1.D0
                  ELSE
                     amag(k)=amag1(k)+conv_mag(k)
                  END IF
               END DO
            ELSE IF (magtyp.eq.2) THEN ! [input in muJy]
               DO k=1,n_filter
                  IF (amag(k).lt.-90.D0) THEN
                     err_p(k)=99.D0
                     amag(k)=mag_nd(k)+conv_mag(k)
                  ELSE IF (ABS(amag(iii)+1.D0).lt.1.D-6) THEN
                     amag(k)=-1.D0
                     err_p(k)=-1.D0
                  ELSE
                     err_p(k)=2.5D0/DLOG(10.D0)*err_p(k)/amag(k)
                     amag(k)=amag1(k)+conv_mag(k)
                  END IF
               END DO
            END IF
            WRITE(14,fmt1) id,(amag(i),i=1,n_filter),
     .           (err_p(i),i=1,n_filter)
         END IF
      END IF

      IF (chi2min.gt.9.9D+11) THEN
         WRITE(*,'(1x,a,i9)') 'WARNING: impossible to find a '//
     .        'solution within constraints for object ', id
c ..... Write outputs in any case 
         r_chi2=99.D0
         qq=0.D0
         i_typ=0
         i_age=0
         age=99.D0
         av=0.D0
         bb=0.D0
         amagb=99.D0
         galmass=-100.D0
         starmass=-100.D0
         sfrgal=-100.D0
         DO i=1,3
            sm1(i)=-100.D0
            sm2(i)=-100.D0
         END DO
         tau_tmp=-100.D0
         DO k=1,n_filter
            amagfit(k)=99.D0
         END DO
         WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .        galmass,starmass,(amagfit(k),k=1,n_filter),amagb,
     .        sm1(1),sm2(1),sm1(2),sm2(2),sm1(3),sm2(3),sfrgal,
     .        tau_tmp,iw_ob
c..... File .zmtemp_sed
         IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
            DO k=1,n_filter
               mag_temp(k)=99.D0
               fl_temp(k)=-1.D0
            END DO
            IF (iunit_sed.eq.0.or.iunit_sed.eq.1) THEN
               WRITE(15,fmt4) id,(fl_temp(k),k=1,n_filter)
            ELSE IF (iunit_sed.eq.2) THEN
               WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filter)
            END IF
         END IF
         GO TO 13400
      END IF

c ..... write solution
      DO k=1,iw(i_typ)
         y(k)=DBLE(f_bc(k,i_age,i_typ))
         ww(k)=DBLE(w_bc(k,i_typ))
      END DO

      av=avmin+(i_red-1)*averr
      IF (av.gt.0.0) THEN
         CALL red_spec(ww,y,av,yob,1,iw(i_typ),law_rd) 
         DO k=1,iw(i_typ)
            y(k)=yob(k)
         END DO
      END IF

      CALL lyman_forest(ww,y,iw(i_typ),zs)

      IF (magtyp.eq.1) THEN
         DO k=1,iw(i_typ)
            yynu(k)=y(k)*ww(k)*ww(k)/c ! f(nu) of template
            wwnu(k)=c/ww(k)     ! frequency of template
         END DO
      END IF
      DO k=1,n_filter
         fluxfit(k)=filter(nfil(k),ww,y,iw(i_typ),zs)
         IF (fluxfit(k).gt.0.D0) THEN
            amagfit(k)=-2.5D0*DLOG10(bb*fluxfit(k)/vega(k))
            IF (magtyp.eq.1) THEN
               l=0
               DO kk=1,np(nfil(k))
                  wrnu(kk)=wrnuf(kk,k) ! frequency of R(nu)
                  rrnu(kk)=rrnuf(kk,k) ! R(nu)
                  flnu(kk)=rrnu(kk)*rlin(wrnu(kk)*(1.+zs),wwnu,yynu,
     &                 iw(i_typ),l)
               END DO
               fluxfitn(k)=trapz1(wrnu,flnu,np(nfil(k)))
     &              *(1.D0+zs)/(f2(k))
               amagfit(k)=-2.5D0*DLOG10(bb*fluxfitn(k))-48.6D0
c               amagfit(k)=-2.5D0*DLOG10(bb*fluxfit(k)*wl(k)*wl(k)
c     &              /(surf(k)*c))-48.6D0
c               amagfit(k)=amagfit(k)+conv_mag(k)
            END IF
         ELSE
            amagfit(k)=99.D0
         END IF
      END DO

      age=t_bc(i_age)*1.D-9
      WRITE(13,fmt) id,zs,r_chi2,qq,i_typ,i_age,age,av,bb,
     .     DLOG10(galmass),DLOG10(starmass),(amagfit(k),k=1,n_filter),
     .     amagb,sm1(1),sm2(1),sm1(2),sm2(2),sm1(3),sm2(3),
     .     sfrgal,tau(i_typ),iw_ob

c file .zmtemp_sed
      IF (csed2.eq.'y'.or.csed2.eq.'Y') THEN 
         DO k=1,n_filter
            fl_temp(k)=filter(nfil(k),ww,y,iw(i_typ),zs)/surf(k)
         END DO
         IF (iunit_sed.eq.0) THEN
            WRITE(15,fmt4) id,(bb*fl_temp(k),k=1,n_filter)
         ELSE IF (iunit_sed.eq.1) THEN
            WRITE(15,fmt4) id,(bb*fl_temp(k)*(wl(k)**2/c)*1.D+29,
     &           k=1,n_filter)
         ELSE IF (iunit_sed.eq.2) THEN
            DO k=1,n_filter
               IF (fl_temp(k).gt.0.D0) THEN
                  mag_temp(k)=-2.5D0*DLOG10(bb*fl_temp(k)*
     .                 surf(k)/vega(k))+conv_mag(k)
               ELSE
                  mag_temp(k)=99.D0
               END IF
            END DO
            WRITE(15,fmt3) id,(mag_temp(k),k=1,n_filter)
         END IF
      END IF
c file .zspe
      IF (iout_s.gt.1) THEN
         spectra=root(1:iroot)//'.zspe'
         IF (iunit_sed.eq.0) THEN
            DO k=1,iw(i_typ)
               y(k)=bb*y(k)/(1.D0+zs)
            END DO
         ELSE IF (iunit_sed.eq.1) THEN
            DO k=1,iw(i_typ)
               y(k)=bb*y(k)*(ww(k)**2/c)*1.D+29*
     .              (1.D0+zs)
            END DO
         ELSE IF (iunit_sed.eq.2) THEN
            DO k=1,iw(i_typ)
               y(k)=bb*y(k)*(ww(k)**2/c)*(1.D0+zs)
               IF (y(k).ne.0.D0) THEN
                  y(k)=-2.5D0*DLOG10(y(k))-48.60D0
               ELSE
                  y(k)=100.D0
               END IF
            END DO
         END IF
         iwav=0
         wmin1=1.D5
         wmax1=0.D0
         DO k=1,n_filter
            wmin0=wl(k)-0.5D0*surf(k)-1000.D0
            wmax0=wl(k)+0.5D0*surf(k)+1000.D0
            IF (wmin0.lt.wmin1) wmin1=wmin0
            IF (wmax0.gt.wmax1) wmax1=wmax0
         END DO
c Uncomment the following 2 lines and select the range 
c of lambda you prefer for the .zspe output
c         wmin1=1000.D0          ! lambda min in Angstrom
c         wmax1=1000000.D0         ! lambda max in Angstrom
         DO k=1,iw(i_typ)
            w0=ww(k)*(1.D0+zs)
            IF ((w0.ge.wmin1.and.w0.le.wmax1).or.
     .           (w0.le.wmin1.and.w0.ge.wmax1)) THEN
               iwav=iwav+1
               wsyn(iwav)=ww(k)*(1.D0+zs)
               ysyn(iwav)=y(k)
            END IF
         END DO
         OPEN(12,FILE=spectra,STATUS='unknown')
         DO k=1,iwav
            WRITE(12,2013) wsyn(k),ysyn(k)
         END DO
         CLOSE(12)
      END IF
c.......................................................................
      GO TO 13400
13500 CONTINUE
c.......................................................................
      CLOSE(16)
      CLOSE(13)
      IF (csed1.eq.'y'.or.csed1.eq.'Y') CLOSE(14)
      IF (csed2.eq.'y'.or.csed2.eq.'Y') CLOSE(15)

      time_tot=DBLE(cputime())-time_tot
      WRITE(*,'(a)') ' Done !!!'
      WRITE(*,'(1X,A,F12.3,A)') 'Total CPU time involved : ', time_tot, 
     .     ' seconds'
      WRITE(18,'(1X,A,F12.3,A)') '#  Total CPU time involved : ',  
     .     time_tot,' seconds'
      CLOSE(18)
      STOP
 2012 FORMAT(i10,1x,f7.4,1x,i3,1x,i3,1x,f8.4,1x,f5.2,1x,E11.5,1X,
     .     F10.5,1X,E9.3,1X,F9.4,1X,F12.6,1X,F12.6,1X,F12.4,1X,F9.4)
 2013 FORMAT(1x,f15.4,4x,e20.13)
 25   STOP 'Error opening synthetic spectra files'
 26   STOP 'Error opening .4color files'
 78   STOP 'Error opening templates file'
 79   STOP 'Error opening catalogue file'
 80   STOP 'Error opening filters file'
 81   STOP 'Error opening output files'
 82   STOP 'Error reading filters file'
      END
c----------------------------------------------------------------------
      FUNCTION func(x)
      IMPLICIT NONE
      REAL*8 func,pchi2,x,deg,prob
      EXTERNAL pchi2
      COMMON/nfree/deg,prob
c
      func=prob-pchi2(0.5D0*deg,0.5D0*x)
      RETURN
      END

c----------------------------------------------------------------------
c ..... Luminosity distance [Mpc]
 
      FUNCTION d_l(h0,om,ov,za)
      IMPLICIT NONE
      INTEGER nstep,i
      REAL*8 h0,om,ov,z,za,c,d_l,ee,s,s1,s2,dz,a,b,x
      PARAMETER (c=2.9979D+5,nstep=100)
      ee(x)=DSQRT(om*(1.D0+x)**3+(1.D0-om-ov)*(1.D0+x)**2+ov)
c
      IF (za.lt.1.D-5) THEN
         z=1.D-5
      ELSE
         z=za
      END IF
      IF (om.lt.0.001D0.and.ov.lt.1.D-8) THEN
         d_l=(1.D0+z)*DLOG(1.D0+z)
      ELSE IF ((om-1.D0).lt.1.D-8.and.ov.lt.1.D-8) THEN
         d_l=2.D0*(1.D0+z)*(1.D0-1.D0/DSQRT(1.D0+z))
      ELSE IF (om.lt.1.D0.and.om.gt.0.01D0.and.ov.lt.1.D-8) THEN
         d_l=(2.D0/om**2)*(om*z-(2.D0-om)*(-1.D0+DSQRT(om*z+1.D0))) ! Mattig
      ELSE
         dz=z/DBLE(FLOAT(nstep))
         s=0.D0
         d_l=0.D0
         DO i=1,nstep+1
            a=(i-1)*dz
            b=i*dz
            s1=(1.D0/ee(a))
            s2=(1.D0/ee(b))
            s=dz*(s1+s2)*0.5D0
            d_l=d_l+s
         END DO
         d_l=(1.D0+z)*d_l
      END IF
      d_l=c/h0*d_l ! luminosity distance in Mpc
      RETURN
      END
c ----------------------------------------------------------------------

      SUBROUTINE sort2(n,arr,brr)
      INTEGER n,M,NSTACK
      REAL*8 arr(n),brr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL*8 a,b,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          b=brr(j)
          do 11 i=j-1,1,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
            brr(i+1)=brr(i)
11        continue
          i=0
2         arr(i+1)=a
          brr(i+1)=b
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        temp=brr(k)
        brr(k)=brr(l+1)
        brr(l+1)=temp
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
          temp=brr(l+1)
          brr(l+1)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
          temp=brr(l)
          brr(l)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l+1).gt.arr(l))then
          temp=arr(l+1)
          arr(l+1)=arr(l)
          arr(l)=temp
          temp=brr(l+1)
          brr(l+1)=brr(l)
          brr(l)=temp
        endif
        i=l+1
        j=ir
        a=arr(l)
        b=brr(l)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        temp=brr(i)
        brr(i)=brr(j)
        brr(j)=temp
        goto 3
5       arr(l)=arr(j)
        arr(j)=a
        brr(l)=brr(j)
        brr(j)=b
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort2'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
C  (C) Copr. 1986-92 Numerical Recipes Software ]2#1.
